/****
 * CFileWndKeys.c
 *
 *	File window key methods for the SignWriter key editor.
 *
 *  Copyright 1995 The Center for Sutton Movement Writing. All rights reserved.
 *
 ****/




#include "CFileWndKeys.h"
#include "CApplication.h"
#include "CBureaucrat.h"
#include "CSelector.h"
#include "CFileWndTask.h"
#include "CFingKeyWindow.h"
#include "CSignKeyWindow.h"
#include "CSymWindow.h"
#include "CWindow.h"
#include "Global.h"
#include "KeyEditConsts.h"
#include "Keys.h"
#include <Script.h>
#include "TBUtilities.h"




/*** Global Variables ***/

extern  void    *gClipRec;							/* Clipboard */
extern  ObjType  gClipType;

extern	void    *gSignKeyWindow[KEY_CODES];			/* Window management 	*/
extern	void    *gFingerKeyWindow[KEY_CODES];

extern	short	 gKeyWindows;





/****************************  Initialization  ****************************/


void CFileWndKeys::IFileWndKeys (CView *anEnclosure, CBureaucrat *aSupervisor,
								  short aHEncl, short aVEncl,
								  short numKeys, Byte keyCodes[],
								  short paneCmd, CFileWndKeys *nextKeyPane)
								  
{
	inherited::IGridSelector(anEnclosure, aSupervisor,
				aHEncl, aVEncl, sizFIXEDSTICKY, sizFIXEDSTICKY,
				SELnone, 0, 1, numKeys,
				FCELL_WIDTH, FCELL_HEIGHT);
				
	CSelector::SetCommandBase(paneCmd);
				
	itsKeyCodes = keyCodes;
	
	itsNextPane = nextKeyPane;
	
	itsCurKbd = sign;				// default is Sign keyboard
	
}					  
    



/****************************  Window management  ****************************/

 
/***
 * GetSignKeyWindow
 *
 *	Return the sign key window corresponding
 *  to the specified key code.
 *
 ***/
 
void CFileWndKeys::GetSignKeyWindow (short keyCode, void **wind)

{
	*wind = gSignKeyWindow[keyCode];
}



/***
 * SetSignKeyWindow
 *
 *	Assign a sign key window to the 
 *  specified key code.
 *
 ***/
 
void CFileWndKeys::SetSignKeyWindow (short keyCode, void *wind)

{
	gSignKeyWindow[keyCode] = wind;

	if (wind != NULL)
		gKeyWindows++;
	else 
		gKeyWindows--;
}


 
/***
 * GetFingerKeyWindow
 *
 *	Return the fingerspell key window corresponding
 *  to the specified key code.
 *
 ***/
 
void CFileWndKeys::GetFingerKeyWindow (short keyCode, void **wind)

{
	*wind = gFingerKeyWindow[keyCode];
}



/***
 * SetFingerKeyWindow
 *
 *	Assign a fingerspell key window to the 
 *  specified key code.
 *
 ***/
 
void CFileWndKeys::SetFingerKeyWindow (short keyCode, void *wind)

{
	gFingerKeyWindow[keyCode] = wind;

	if (wind != NULL)
		gKeyWindows++;
	else 
		gKeyWindows--;
}



/***
 * KeyWindowsOpen
 *
 *	Check for open key box windows.
 *
 ***/
 
Boolean CFileWndKeys::KeyWindowsOpen()

{
	short junk;
	
	if (gKeyWindows > 0) {

		PositionDialog('ALRT', ALRTopenKeys);	
		junk = Alert(ALRTopenKeys, NULL);	
		
		return TRUE;
	}
	
	return FALSE;	
}




 /******************************  Grid mapping  *******************************/



/***
 * ItemToKeyCode
 *
 *	Return the key code that corresponds  
 *  to the current grid pane item. 
 *
 ***/

 Byte CFileWndKeys::ItemToKeyCode (short item)
 
 {
 	item--;
 	return itsKeyCodes[item];
 }
 
 
 
/***
 * ItemToCharCode
 *
 *	Return the char value that corresponds  
 *  to the current grid pane item. 
 *
 ***/

 char CFileWndKeys::ItemToCharCode (short item)
 
 {
 	return KeyToCharCode(ItemToKeyCode(item));
 }

 
 
/***
 * KeyToCharCode
 *
 *	Return the char value that corresponds  
 *  to the specified key code. 
 *
 ***/

 char CFileWndKeys::KeyToCharCode (short keyCode)
 
 {
 	Ptr   	KCHRPtr = (Ptr) GetEnvirons(smKCHRCache);
	long	state = 0;
	char	code;
	
 	if (KCHRPtr) {
 	
		code = (char) LoByte(KeyTrans(KCHRPtr, keyCode, &state));	
	 	UpperText(&code, sizeof code);
	 	return code;

	} else
 		return '?';

 }



/***
 * KeyName
 *
 *	Return string containing the
 *  name of the key that corresponds
 *  to the specified command value.
 *
 ***/
 
 unsigned char *CFileWndKeys::KeyName (long theCommand)
 
 {
 	static char str[2] = " ";
 	
 	if (HiShort(-theCommand) != CSelector::GetCommandBase() && itsNextPane)
 		return itsNextPane->KeyName(theCommand);
 		
 	str[0] = ItemToCharCode(LoShort(-theCommand));
	return ((unsigned char *) &str[0]);

 }
 
 
 
 
 /*******************************  Selection  ********************************/


 /***
 * UpdateSelect
 *
 *	Deselect all key panes other than the
 *  the one specified by the command value.
 *  This ensures that only one key selection
 *  exists at a time in the keyboard display.
 *
 ***/
 
 void CFileWndKeys::UpdateSelect (long theCommand)
 
 {	
 	Prepare();
 	
 	if (HiShort(-theCommand) != CSelector::GetCommandBase())
 		CSelector::ChangeSelection(SELnone);
 	
 	if (itsNextPane)
 		itsNextPane->UpdateSelect(theCommand);
 		
 }
 
 
 
 
 /********************************  Drawing  ********************************/


/***
 * DrawItem
 *
 *  Draw the symbols contained in the specified grid item. 
 *  The item numbering scheme is as defined for CGridSelector.
 *
 ***/
 
void CFileWndKeys::DrawItem (short theItem, Rect *theBox)
 
{
	Byte  keyCode = ItemToKeyCode(theItem);
	
	
	EraseRect(theBox);	

	if (itsCurKbd == sign)
		DrawSignKey(keyCode, 0, &thePort->portBits, theBox, TRUE);
				
	else /* fingerspell */
		DrawFingerKey(keyCode, &thePort->portBits, theBox, TRUE);
		
}



/***
 * UpdateKey
 *
 *  Update the specified grid
 *  item after key editing.
 *
 ***/

void CFileWndKeys::UpdateKey (Byte keyCode)
 
{
	short	inx, item;
 	Rect	box;
	
	
 	for (inx = 0; inx < inherited::columns; inx++)		
 		if (keyCode == itsKeyCodes[inx]) {
 			
 			item = inx + 1;
 			
			Prepare();
			FindItemBox(item, &box);
			DrawItem(item, &box);	
			
			if (item == CSelector::GetSelection())
				HiliteItem (item, hiliteON);
				
			return;			
		}
		
	if (itsNextPane)
 		itsNextPane->UpdateKey(keyCode);
 		
}



/***
 * Update
 *
 *  Update the keyboard display.
 *
 ***/

void CFileWndKeys::Update(KbdType theKbd)

{
	LongRect	theFrame;
	Rect		theRect;
	
	itsCurKbd = theKbd;
	
	Prepare();
	
	GetFrame(&theFrame);
	FrameToQDR(&theFrame, &theRect);
	
	inherited::Draw(&theRect);
	
	if (itsNextPane)
 		itsNextPane->Update(theKbd);
 
 }




/********************************  Clipboard  ********************************/


/***
 * GetKey
 *
 *  Return record corresponding to the specified key.
 *
 ***/
 
void CFileWndKeys::GetKey (Byte keyCode, void **rec)

{	
	if (itsCurKbd == sign)
		GetSignKey(keyCode, (BoxRow *) rec);
	
	else /* fingerspell */
		GetFingerKey(keyCode, (Box *) rec);
		
}
				  


/***
 * SetKey
 *
 *  Update record corresponding to the specified key.
 *
 ***/
 
void CFileWndKeys::SetKey (Byte keyCode, void *rec)

{
	if (itsCurKbd == sign)
		SetSignKey(keyCode, (BoxRow) rec);
	
	else /* fingerspell */
		SetFingerKey(keyCode, (Box) rec);
		
}
		  

		
/***
 * CopyKey
 *
 *  Copy the specified key record.
 *
 ***/
 
void CFileWndKeys::CopyKey (void *rec, void **copyRec)

{
	if (itsCurKbd == sign)
		CopySignKey((BoxRow) rec, (BoxRow *) copyRec);
	
	else /* fingerspell */
		CopyFingerKey((Box) rec, (Box *) copyRec);
		
}
					  

		
/***
 * ClearKey
 *
 *  Clear the specified key record.
 *
 ***/
 
void CFileWndKeys::ClearKey (void **rec)

{
	if (itsCurKbd == sign)
		ClearSignKey((BoxRow) rec);
	
	else /* fingerspell */
		ClearFingerKey((Box) rec);
			
}
				  


/***
 * GetKeyType
 *
 *  Return the specified key record type.
 *
 ***/
 
void CFileWndKeys::GetKeyType (ObjType *type)

{
	*type = (itsCurKbd == sign) ? key : box;		
}			




/********************************  Commands  *******************************/

	
/***
/* 
/*  DoCopy
/*
/*  Copy selected item to the clipboard. Not undoable.
/*
 ***/
		
void CFileWndKeys::DoCopy()

{
	short	item = CSelector::GetSelection();
	Byte	keyCode;
	void	*rec;
	
	if (item == SELnone && itsNextPane) {
	
		itsNextPane->DoCopy();
		return;
	}
	
	keyCode = ItemToKeyCode(item);
	
	if (gClipRec != NULL)
		DisposKey(gClipType, gClipRec);
	
	GetKey(keyCode, &rec);
	CopyKey(rec, &gClipRec);
		
	GetKeyType(&gClipType);
	
}
	


/***
/* 
/*  DoClear
/*
/*  Erase the currently selected item
/*
 ***/

void CFileWndKeys::DoClear (short taskIndex)

{
	short		    item = CSelector::GetSelection();
	Byte			keyCode;
	CFileWndClear   *theClearTask;
	
	if (KeyWindowsOpen())
		return;
		
	if (item == SELnone && itsNextPane) {
	
		itsNextPane->DoClear(taskIndex);
		return;
	}
	
	keyCode = ItemToKeyCode(item);
					
	theClearTask = new CFileWndClear;
	theClearTask->IFileWndClear(taskIndex, this, keyCode);
	theClearTask->Do();
	
	itsSupervisor->Notify(theClearTask);
	itsLastTask = theClearTask;
	
}



/***
/* 
/*  DoPaste
/*
/*  Paste key record from the clipboard.
/*
 ***/

void CFileWndKeys::DoPaste()

{
	short		    item = CSelector::GetSelection();
	Byte 			keyCode;
	CFileWndPaste   *thePasteTask;
	
	if (KeyWindowsOpen())
		return;
			
	if (item == SELnone && itsNextPane) {
	
		itsNextPane->DoPaste();
		return;
	}		
	
	keyCode = ItemToKeyCode(item);
							
	thePasteTask = new CFileWndPaste;
	thePasteTask->IFileWndPaste(this, keyCode);
	thePasteTask->Do();
	
	itsSupervisor->Notify(thePasteTask);
	itsLastTask = thePasteTask;
	
}	



/***
 * DoKeyCommand
 *
 *	Open the key box window corresponding to the
 *  specified key code.  Do not change the current
 *  key pane selection.
 *
 ***/
 
void CFileWndKeys::DoKeyCommand (Byte keyCode)
 
{
 	short	inx;

 	for (inx = 0; inx < inherited::columns; inx++)		
 		if (keyCode == itsKeyCodes[inx]) {
 			
			OpenKeyWindow(keyCode, KeyToCharCode(keyCode));
			return;			
		}
		
	if (itsNextPane)
 		itsNextPane->DoKeyCommand(keyCode);
 		
}
 		



/**********************************  Mouse  ***********************************/


/***
 * DoDoubleClick (override default)
 *
 * 	Open the selected key for editing.
 *
 ***/

void CFileWndKeys::DoDoubleClick (void)

{
	short item = CSelector::GetSelection();
	
	OpenKeyWindow(ItemToKeyCode(item), ItemToCharCode(item));
}




/******************************  Key editing  *******************************/


/***
 * OpenKeyWindow
 *
 *	Open edit window for the specified key.
 *
 ***/

void CFileWndKeys::OpenKeyWindow (Byte keyCode, char keyName)

{
	void	*keyWind;

	if (itsCurKbd == sign) {
		
		GetSignKeyWindow(keyCode, &keyWind);	// window already open?
	
		if (keyWind != NULL)
		  ((CSignKeyWindow *) keyWind)->itsWindow->Select();
		else		
		  NewSignKeyWindow(keyCode, keyName);
	
	
	} else {  	// fingerspell
	
		GetFingerKeyWindow(keyCode, &keyWind);
	
		if (keyWind != NULL)
		  ((CFingKeyWindow *) keyWind)->itsWindow->Select();
		else		
		  NewFingerKeyWindow(keyCode, keyName);	  
	}	
	
}	
	
	
	
/***
 * NewSignKeyWindow
 *
 *	Create new instance of a sign key window
 *	and send it a NewWindow message.
 *
 ***/

void CFileWndKeys::NewSignKeyWindow (Byte keyCode, char keyName)

{
  	CSignKeyWindow	*theWindow = NULL;

	TRY
	{
		theWindow = new CSignKeyWindow;
		
		theWindow->ISignKeyWindow((CDirectorOwner*) this->itsSupervisor,
								  keyCode, keyName);
	
		theWindow->BuildWindow();
	}
	
	CATCH
	{
		 if (theWindow) theWindow->Dispose();
	}
	ENDTRY;
}
	
	
	
/***
 * NewFingerKeyWindow
 *
 *	Create new instance of a fingerspelling key
 *	window and send it a NewWindow message.
 *
 ***/

void CFileWndKeys::NewFingerKeyWindow (Byte keyCode, char keyName)

{
  	CFingKeyWindow	*theWindow = NULL;

	TRY
	{
		theWindow = new CFingKeyWindow;
		
		theWindow->IFingKeyWindow((CDirectorOwner*) this->itsSupervisor,
								  keyCode, keyName);

		theWindow->BuildWindow();
	}
	
	CATCH
	{
		 if (theWindow) theWindow->Dispose();
	}
	ENDTRY;
}

