
/****
 * CSymWndDialog.c
 *
 *	Symbol window dialog methods for the SignWriter Image editor.
 *
 *  Copyright  1995 The Center for Sutton Movement Writing.  All rights reserved.
 *
 ****/



#include "CSymWndDialog.h"
#include "CApplication.h"
#include "CCheckBox.h"
#include "CControl.h"
#include "CSymWindow.h"
#include "CWindow.h"
#include "Commands.h"
#include "SymEditConsts.h"
#include "TBUtilities.h"





/**************************  Common dialog methods  ****************************/


/***
 * ISymWndDialog
 *
 *  Initialize a new instance of a symbol window dialog.
 *
 ***/

void CSymWndDialog::ISymWndDialog (short DLOGid, CDirectorOwner *aSupervisor)

{	
	ImageClass	imgClass;
	short		rots, flops;
	
	// do inherited
	CDLOGDirector::IDLOGDirector(DLOGid, aSupervisor);
	
	
	// get sym window reference
 	itsSymWind = ((CSymWindow*) aSupervisor)->itsGridPane;


	// get selected fill variation
	itsFillVar = itsSymWind->GetSelection();
	if (itsFillVar < 0) itsFillVar = -itsFillVar;  // normalize column selection
				
	// get current attributes
	GetClass(itsSymWind->itsSymCode, itsSymWind->itsSymVar, itsFillVar, &imgClass);
	GetRotations(itsSymWind->itsSymCode, itsSymWind->itsSymVar, itsFillVar, &rots);
	GetFlops(itsSymWind->itsSymCode, itsSymWind->itsSymVar, itsFillVar, &flops);
	
	// init class vars
	itsClass = imgClass;
	itsRots  = rots;
	itsFlops = flops;					     
}

 
 
/***
 * GetNewAttrs
 *
 *  Perform modal dialog. Return the new fill
 *  attribute values and the terminating command.
 *
 ***/

long CSymWndDialog::GetNewAttrs (ImageClass *imgClass, short *rots, short *flops)

{
	long  finishCmd;
		        
	// decode attribute values into initial checkbox settings	
	AttrsToCheckBoxes();
	
	// perform dialog
	BeginDialog();	
	finishCmd = DoModalDialog(cmdOK);
	
	// return new attribute values
	*imgClass = itsClass;
	*rots     = itsRots;
	*flops    = itsFlops;
	
	// return terminating command
	return finishCmd;
}

	
			
/***
 * Validate (override)
 *
 *
 ***/

Boolean CSymWndDialog::Validate (void)

{
	ImageClass  curClass;
	short	  curImgs, newImgs;
	
	// do inherited
	if (!inherited::Validate())
		return FALSE;
				 
	// encode final checkbox settings into new attribute values
	CheckBoxesToAttrs();
	
	// get current symbol class 
	GetClass(itsSymWind->itsSymCode, itsSymWind->itsSymVar, itsFillVar, &curClass);
	
	// get # images for current and new classes
	GetImages(curClass, &curImgs);
	GetImages(itsClass, &newImgs);
	
	if (newImgs < curImgs) {
		PositionDialog('ALRT', ALRTloseImgs);	
		return (CautionAlert(ALRTloseImgs, NULL) == OK);
	
	} else
		return TRUE;
}



/***
 * AttrsToCheckBoxes
 *
 *  Decode attributes into checkbox settings
 *	
 ***/
 
void CSymWndDialog::AttrsToCheckBoxes (void)

{
	/*  null method  */
}	



/***
 * CheckBoxesToAttrs
 *
 *	Encode checkbox settings into attributes
 *
 ***/
 
void CSymWndDialog::CheckBoxesToAttrs (void)

{
	/*  null method  */
}	




/**************************  Hand dialog methods  ****************************/


/***
 * IHandDialog
 *
 *  Initialize a new instance of a hand attributes dialog.
 *
 ***/

void CHandDialog::IHandDialog (CDirectorOwner *aSupervisor)

{	
	Str27	winTitle = "\pHand Symbol Attributes";


	// do inherited
	
	CSymWndDialog::ISymWndDialog(DLOGhandAttrs, aSupervisor);
			     
    
	// Translate dialog resource item numbers into checkbox objects
	
	symVert = (CCheckBox*) itsWindow->FindViewByID (DITLhandSymVert);
	symHorz = (CCheckBox*) itsWindow->FindViewByID (DITLhandSymHorz);
	
	ident45   = (CCheckBox*) itsWindow->FindViewByID (DITLhandIdent45 );
	ident90   = (CCheckBox*) itsWindow->FindViewByID (DITLhandIdent90 );
	ident180  = (CCheckBox*) itsWindow->FindViewByID (DITLhandIdent180);
	
	
	// Assign local command numbers to checkbox objects

	symVert->SetClickCmd (DITLhandSymVert);
	symHorz->SetClickCmd (DITLhandSymHorz);
	
	ident45  ->SetClickCmd (DITLhandIdent45 );
	ident90  ->SetClickCmd (DITLhandIdent90 );
	ident180 ->SetClickCmd (DITLhandIdent180);
	
	
	// Set window title
	
	itsWindow->SetTitle(winTitle);
	
}
	
	
		
/***
 * DoCommand (override)
 *
 *  Maintain checkbox consistency during dialog.
 *
 ***/

void CHandDialog::DoCommand (long theCommand)

{
	switch (theCommand) {
			
		case DITLhandSymHorz:
		case DITLhandSymVert:

			if (symHorz->IsChecked() == symVert->IsChecked()) {
			
				if (symHorz->IsChecked()) {
					ident45->Activate();
					ident90->Activate();
				}
				ident180->Activate();
					
			} else {		
				ident45 ->SetValue(BUTTON_OFF);
				ident90 ->SetValue(BUTTON_OFF);
				ident180->SetValue(BUTTON_OFF);
				
				ident45->Deactivate();
				ident90->Deactivate();
				ident180->Deactivate();
			}		
			break;
		
		
		case DITLhandIdent45: 
			if (ident45->IsChecked()) {
				ident90 ->SetValue(BUTTON_OFF);
				ident180->SetValue(BUTTON_OFF);
			}
			break;
				
				
		case DITLhandIdent90: 
			if (ident90->IsChecked()) {
				ident45 ->SetValue(BUTTON_OFF);
				ident180->SetValue(BUTTON_OFF);
			}
			break;
			
			
		case DITLhandIdent180: 
			if (ident180->IsChecked()) {
				ident45 ->SetValue(BUTTON_OFF);
				ident90 ->SetValue(BUTTON_OFF);
			}break;	
			
				
		default:
			inherited::DoCommand(theCommand);
			break;
	}
}



/***
 * AttrsToCheckBoxes (override)
 *
 *  Decode attribute values into checkbox settings
 *	
 ***/
 
void CHandDialog::AttrsToCheckBoxes (void)

{
	switch (itsClass) {
	
		case class1:
			symVert->SetValue(BUTTON_ON );
			symHorz->SetValue(BUTTON_ON );
			
			ident45  ->SetValue(BUTTON_ON );
			ident90  ->SetValue(BUTTON_OFF);
			ident180 ->SetValue(BUTTON_OFF);
			
			break;
			
			
		case class2:
			symVert->SetValue(BUTTON_ON );
			symHorz->SetValue(BUTTON_ON );
			
			ident45  ->SetValue(BUTTON_OFF);
			ident90  ->SetValue(BUTTON_ON );
			ident180 ->SetValue(BUTTON_OFF);
			
			break;
			
			
		case class3A:
			if (itsRots == 3) {
				symVert->SetValue(BUTTON_ON );
				symHorz->SetValue(BUTTON_ON );
				
				ident45  ->SetValue(BUTTON_OFF);
				ident90  ->SetValue(BUTTON_OFF);
				ident180 ->SetValue(BUTTON_ON );


			} else /* rots = 7 */ {
				symVert->SetValue(BUTTON_ON );
				symHorz->SetValue(BUTTON_OFF);
				
				ident45  ->SetValue(BUTTON_OFF);
				ident90  ->SetValue(BUTTON_OFF);
				ident180 ->SetValue(BUTTON_OFF);
				
				ident45  ->Deactivate();
				ident90  ->Deactivate();
				ident180 ->Deactivate();
			}
			break;
	
	
		case class3B:
				symVert->SetValue(BUTTON_OFF);
				symHorz->SetValue(BUTTON_ON );
				
				ident45  ->SetValue(BUTTON_OFF);
				ident90  ->SetValue(BUTTON_OFF);
				ident180 ->SetValue(BUTTON_OFF);
					
				ident45  ->Deactivate();
				ident90  ->Deactivate();
				ident180 ->Deactivate();
				break;		
		
			
		case class4:
			if (itsRots == 3) {
				symVert->SetValue(BUTTON_OFF);
				symHorz->SetValue(BUTTON_OFF);
				
				ident45  ->SetValue(BUTTON_OFF);
				ident90  ->SetValue(BUTTON_OFF);
				ident180 ->SetValue(BUTTON_ON);
				
				ident45  ->Deactivate();
				ident90  ->Deactivate();
					
					
			} else /* rots = 7 */ {
				symVert->SetValue(BUTTON_OFF);
				symHorz->SetValue(BUTTON_OFF);
				
				ident45  ->SetValue(BUTTON_OFF);
				ident90  ->SetValue(BUTTON_OFF);
				ident180 ->SetValue(BUTTON_OFF);
			
				ident45  ->Deactivate();
				ident90  ->Deactivate();
			}
			break;
			
			
		default:
			symVert->SetValue(BUTTON_OFF);
			symHorz->SetValue(BUTTON_OFF);
			
			ident45  ->SetValue(BUTTON_OFF);
			ident90  ->SetValue(BUTTON_OFF);
			ident180 ->SetValue(BUTTON_OFF);
			
			ident45  ->Deactivate();
			ident90  ->Deactivate();
			break;		
	}
}	
	
	

/***
 * CheckBoxesToAttrs (override)
 *
 *	Encode checkbox settings into attribute values
 *
 ***/
 
void CHandDialog::CheckBoxesToAttrs (void)

{
	if (symVert->IsChecked() && symHorz->IsChecked()) {
	
		if (ident45->IsChecked()) {
			itsClass = class1;
			itsRots  = 0;
			itsFlops = 0;
			
		} else if (ident90->IsChecked()) {
			itsClass = class2;
			itsRots  = 1;
			itsFlops = 0;
			
		} else /* ident180 default */ {
			itsClass = class3A;
			itsRots  = 3;
			itsFlops = 0;	
		}
		
	
	} else if (symVert->IsChecked()) {
			itsClass = class3A;
			itsRots  = 7;
			itsFlops = 0;	
			
	
	} else if (symHorz->IsChecked()) {
			itsClass = class3B;
			itsRots  = 7;
			itsFlops = 0;	
	
	
	} else /* neither */ {
		
		if (ident180->IsChecked()) {
			itsClass = class4;
			itsRots  = 3;
			itsFlops = 1;
			
		} else {
			itsClass = class4;
			itsRots  = 7;
			itsFlops = 1;	
		}
	}		
}




/**************************  Movement dialog methods  *************************/


/***
 * IMvmtDialog
 *
 *  Initialize a new instance of a movement attributes dialog.
 *
 ***/

void CMvmtDialog::IMvmtDialog (CDirectorOwner *aSupervisor)

{	
	Str27	winTitle = "\pMovement Symbol Attributes";


	// do inherited
	
	CSymWndDialog::ISymWndDialog(DLOGmvmtAttrs, aSupervisor);
			     
    
	// Translate dialog resource item numbers into checkbox objects
	
	symVert = (CCheckBox*) itsWindow->FindViewByID (DITLmvmtSymVert);
	symHorz = (CCheckBox*) itsWindow->FindViewByID (DITLmvmtSymHorz);
	
	ident45  = (CCheckBox*) itsWindow->FindViewByID (DITLmvmtIdent45 );
	ident90  = (CCheckBox*) itsWindow->FindViewByID (DITLmvmtIdent90 );
	ident180 = (CCheckBox*) itsWindow->FindViewByID (DITLmvmtIdent180);
	
	vertRot  = (CCheckBox*) itsWindow->FindViewByID (DITLmvmtVertRot);

	
	// Assign local command numbers to checkbox objects

	symVert->SetClickCmd (DITLmvmtSymVert);
	symHorz->SetClickCmd (DITLmvmtSymHorz);
	
	ident45 ->SetClickCmd (DITLmvmtIdent45 );
	ident90 ->SetClickCmd (DITLmvmtIdent90 );
	ident180->SetClickCmd (DITLmvmtIdent180);
		
	vertRot->SetClickCmd (DITLmvmtVertRot);

	
	// Set window title
	
	itsWindow->SetTitle(winTitle);
	
}
	
	
		
/***
 * DoCommand (override)
 *
 *  Maintain checkbox consistency during dialog.
 *
 ***/

void CMvmtDialog::DoCommand (long theCommand)

{
	switch (theCommand) {
			
		case DITLmvmtSymHorz:
		case DITLmvmtSymVert:

			if (symHorz->IsChecked()) {
				vertRot->SetValue(BUTTON_OFF);
				vertRot->Deactivate();
				
			} else {
				vertRot->Activate();
			}
			
			if (symHorz->IsChecked() == symVert->IsChecked()) {
			
				if (symHorz->IsChecked()) {
					ident45->Activate();
					ident90->Activate();
				}	
				ident180->Activate();
					
			} else {		
				ident45 ->SetValue(BUTTON_OFF);
				ident90 ->SetValue(BUTTON_OFF);
				ident180->SetValue(BUTTON_OFF);
				
				ident45->Deactivate();
				ident90->Deactivate();
				ident180->Deactivate();
			}		
			break;
		
		
		case DITLhandIdent45: 
			if (ident45->IsChecked()) {
				ident90 ->SetValue(BUTTON_OFF);
				ident180->SetValue(BUTTON_OFF);
			}
			break;
				
				
		case DITLhandIdent90: 
			if (ident90->IsChecked()) {
				ident45 ->SetValue(BUTTON_OFF);
				ident180->SetValue(BUTTON_OFF);
			}
			break;
			
			
		case DITLhandIdent180: 
			if (ident180->IsChecked()) {
				ident45->SetValue(BUTTON_OFF);
				ident90->SetValue(BUTTON_OFF);
				vertRot->SetValue(BUTTON_OFF);
			}
			break;	
			
			
		case DITLmvmtVertRot: 
			if (vertRot->IsChecked()) {
				ident180->SetValue(BUTTON_OFF);
				ident180->Deactivate();
			}
			break;	


		default:
			inherited::DoCommand(theCommand);
			break;
	}
}



/***
 * AttrsToCheckBoxes (override)
 *
 *  Decode attribute values into checkbox settings
 *	
 ***/
 
void CMvmtDialog::AttrsToCheckBoxes (void)

{
	switch (itsClass) {
	
		case class1:
			if (itsRots == 1) {
						
				if (itsFlops == 1) {
					symVert->SetValue(BUTTON_OFF);
					symHorz->SetValue(BUTTON_OFF);

					ident45 ->SetValue(BUTTON_OFF);
					ident90 ->SetValue(BUTTON_OFF);
					ident180->SetValue(BUTTON_OFF);
										
					vertRot->SetValue(BUTTON_ON );

					ident45->Deactivate();
					ident90->Deactivate();
					
					
				} else /* flops = 0 */ {
					symVert->SetValue(BUTTON_ON );
					symHorz->SetValue(BUTTON_OFF);

					ident45 ->SetValue(BUTTON_OFF);
					ident90 ->SetValue(BUTTON_OFF);
					ident180->SetValue(BUTTON_OFF);
										
					vertRot->SetValue(BUTTON_ON );

					ident45 ->Deactivate();
					ident90 ->Deactivate();
					ident180->Deactivate();
				}
				
				
			} else /* rots = 0 */ {
				symVert->SetValue(BUTTON_ON );
				symHorz->SetValue(BUTTON_ON );	
				
				ident45 ->SetValue(BUTTON_ON );
				ident90 ->SetValue(BUTTON_OFF);
				ident180->SetValue(BUTTON_OFF);
								
				vertRot->SetValue(BUTTON_OFF);

				vertRot->Deactivate();
			}
			break;
			
			
		case class2:
			symVert->SetValue(BUTTON_ON );
			symHorz->SetValue(BUTTON_ON );
			
			ident45  ->SetValue(BUTTON_OFF);
			ident90  ->SetValue(BUTTON_ON );
			ident180 ->SetValue(BUTTON_OFF);
			
			vertRot->SetValue(BUTTON_OFF);

			vertRot->Deactivate();
			
			break;
			
			
		case class3A:
			if (itsRots == 3) {
				symVert->SetValue(BUTTON_ON );
				symHorz->SetValue(BUTTON_ON );
				
				ident45  ->SetValue(BUTTON_OFF);
				ident90  ->SetValue(BUTTON_OFF);
				ident180 ->SetValue(BUTTON_ON );
			
				vertRot->SetValue(BUTTON_OFF);
	
				vertRot ->Deactivate();


			} else /* rots = 7 */ {
				symVert->SetValue(BUTTON_ON );
				symHorz->SetValue(BUTTON_OFF);
				
				ident45  ->SetValue(BUTTON_OFF);
				ident90  ->SetValue(BUTTON_OFF);
				ident180 ->SetValue(BUTTON_OFF);
				
				vertRot->SetValue(BUTTON_OFF);

				ident45  ->Deactivate();
				ident90  ->Deactivate();
				ident180 ->Deactivate();
			}
			break;
	
	
		case class3B:
				symVert->SetValue(BUTTON_OFF);
				symHorz->SetValue(BUTTON_ON );
				
				ident45  ->SetValue(BUTTON_OFF);
				ident90  ->SetValue(BUTTON_OFF);
				ident180 ->SetValue(BUTTON_OFF);
				
				vertRot->SetValue(BUTTON_OFF);
	
				ident45  ->Deactivate();
				ident90  ->Deactivate();
				ident180 ->Deactivate();
				vertRot  ->Deactivate();
				
				break;		
		
			
		case class4:
			if (itsRots == 3) {
				symVert->SetValue(BUTTON_OFF);
				symHorz->SetValue(BUTTON_OFF);
				
				ident45  ->SetValue(BUTTON_OFF);
				ident90  ->SetValue(BUTTON_OFF);
				ident180 ->SetValue(BUTTON_ON);
			
				vertRot->SetValue(BUTTON_OFF);

				ident45  ->Deactivate();
				ident90  ->Deactivate();
					
					
			} else /* rots = 7 */ {
				symVert->SetValue(BUTTON_OFF);
				symHorz->SetValue(BUTTON_OFF);
				
				ident45  ->SetValue(BUTTON_OFF);
				ident90  ->SetValue(BUTTON_OFF);
				ident180 ->SetValue(BUTTON_OFF);
				
				vertRot->SetValue(BUTTON_OFF);

				ident45  ->Deactivate();
				ident90  ->Deactivate();
			}
			break;
			
			
		default:
			symVert->SetValue(BUTTON_OFF);
			symHorz->SetValue(BUTTON_OFF);
			
			ident45  ->SetValue(BUTTON_OFF);
			ident90  ->SetValue(BUTTON_OFF);
			ident180 ->SetValue(BUTTON_OFF);
			
			vertRot->SetValue(BUTTON_OFF);

			ident45  ->Deactivate();
			ident90  ->Deactivate();
			break;		
	}
}	
	
	

/***
 * CheckBoxesToAttrs (override)
 *
 *	Encode checkbox settings into attribute values
 *
 ***/
 
void CMvmtDialog::CheckBoxesToAttrs (void)

{
	if (symVert->IsChecked() && symHorz->IsChecked()) {
	
		if (ident45->IsChecked()) {
			itsClass = class1;
			itsRots  = 0;
			itsFlops = 0;
			
		} else if (ident90->IsChecked()) {
			itsClass = class2;
			itsRots  = 1;
			itsFlops = 0;
			
		} else /* ident180 default */ {
			itsClass = class3A;
			itsRots  = 3;
			itsFlops = 0;	
		}
		
	
	} else if (symVert->IsChecked()) {
	
		if (vertRot->IsChecked()) {
			itsClass = class1;
			itsRots  = 1;
			itsFlops = 0;	
		
		} else {
			itsClass = class3A;
			itsRots  = 7;
			itsFlops = 0;	
		}
			
			
	} else if (symHorz->IsChecked()) {
			itsClass = class3B;
			itsRots  = 7;
			itsFlops = 1;	
	
	
	} else /* neither */ {
		
		if (vertRot->IsChecked()) {
			itsClass = class1;
			itsRots  = 1;
			itsFlops = 1;	
		
		} else if (ident180->IsChecked()) {
			itsClass = class4;
			itsRots  = 3;
			itsFlops = 1;
			
		} else {
			itsClass = class4;
			itsRots  = 7;
			itsFlops = 1;	
		}
	}		
}




/**************************  Face dialog methods  ****************************/


/***
 * IFaceDialog
 *
 *  Initialize a new instance of a hand attributes dialog.
 *
 ***/

void CFaceDialog::IFaceDialog (CDirectorOwner *aSupervisor)

{
	Str27	winTitle = "\pFace Symbol Attributes";
	
	
	// do inherited
	
	CSymWndDialog::ISymWndDialog(DLOGfaceAttrs, aSupervisor);
			     
    
	// Translate dialog resource item numbers into checkbox objects
	
	symVert = (CCheckBox*) itsWindow->FindViewByID (DITLfaceSymVert);
	
	express1 = (CCheckBox*) itsWindow->FindViewByID (DITLfaceExpress1);
	express2 = (CCheckBox*) itsWindow->FindViewByID (DITLfaceExpress2);
	express3 = (CCheckBox*) itsWindow->FindViewByID (DITLfaceExpress3);
	express7 = (CCheckBox*) itsWindow->FindViewByID (DITLfaceExpress7);

	contact7 = (CCheckBox*) itsWindow->FindViewByID (DITLfaceContact7);


	// Assign local command numbers to checkbox objects

	symVert->SetClickCmd (DITLfaceSymVert);
	
	express1->SetClickCmd (DITLfaceExpress1);
	express2->SetClickCmd (DITLfaceExpress2);
	express3->SetClickCmd (DITLfaceExpress3);
	express7->SetClickCmd (DITLfaceExpress7);
	
	contact7->SetClickCmd (DITLfaceContact7);
	
	
	// Set window title
	
	itsWindow->SetTitle(winTitle);
	
}
	
	
		
/***
 * DoCommand (override)
 *
 *  Maintain checkbox consistency during dialog.
 *
 ***/

void CFaceDialog::DoCommand (long theCommand)

{
	switch (theCommand) {
			
		case DITLfaceSymVert:
			if (symVert->IsChecked()) {
				express7->Activate();
				contact7->Activate();
			
			} else {
				express7->SetValue(BUTTON_OFF);
				contact7->SetValue(BUTTON_OFF);

				express7->Deactivate();
				contact7->Deactivate();
			}
			break;
			
		
		case DITLfaceExpress1: 
			if (express1->IsChecked()) {
				express2->SetValue(BUTTON_OFF);
				express3->SetValue(BUTTON_OFF);
				express7->SetValue(BUTTON_OFF);
				contact7->SetValue(BUTTON_OFF);
			}
			break;
			
		
		case DITLfaceExpress2: 
			if (express2->IsChecked()) {
				express1->SetValue(BUTTON_OFF);
				express3->SetValue(BUTTON_OFF);
				express7->SetValue(BUTTON_OFF);
				contact7->SetValue(BUTTON_OFF);
			}	
			break;
			
		
		case DITLfaceExpress3: 
			if (express3->IsChecked()) {
				express1->SetValue(BUTTON_OFF);
				express2->SetValue(BUTTON_OFF);
				express7->SetValue(BUTTON_OFF);
				contact7->SetValue(BUTTON_OFF);
			}	
			break;
			
		
		case DITLfaceExpress7: 
			if (express7->IsChecked()) {
				express1->SetValue(BUTTON_OFF);
				express2->SetValue(BUTTON_OFF);
				express3->SetValue(BUTTON_OFF);
				contact7->SetValue(BUTTON_OFF);
			}	
			break;
			
		
		case DITLfaceContact7: 
			if (contact7->IsChecked()) {
				express1->SetValue(BUTTON_OFF);
				express2->SetValue(BUTTON_OFF);
				express3->SetValue(BUTTON_OFF);
				express7->SetValue(BUTTON_OFF);
			}
			break;
			
	
		default:
			inherited::DoCommand(theCommand);
			break;
	}
}



/***
 * AttrsToCheckBoxes (override)
 *
 *  Decode attribute values into checkbox settings
 *	
 ***/
 
void CFaceDialog::AttrsToCheckBoxes (void)

{
	switch (itsClass) {
	
		case class1:
			if (itsFlops == 1) {
				symVert->SetValue(BUTTON_OFF);
				
				express1->SetValue(BUTTON_OFF);
				express2->SetValue(BUTTON_OFF);
				express3->SetValue(BUTTON_OFF);
				express7->SetValue(BUTTON_OFF);

				contact7->SetValue(BUTTON_OFF);
			
			
			} else /* flops = 0 */ {
				symVert->SetValue(BUTTON_ON );
				
				express1->SetValue(BUTTON_OFF);
				express2->SetValue(BUTTON_OFF);
				express3->SetValue(BUTTON_OFF);
				express7->SetValue(BUTTON_OFF);

				contact7->SetValue(BUTTON_OFF);
			}
			break;
			
			
		case class2:
			if (itsFlops == 1) {
				symVert->SetValue(BUTTON_OFF);
				
				express1->SetValue(BUTTON_ON );
				express2->SetValue(BUTTON_OFF);
				express3->SetValue(BUTTON_OFF);
				express7->SetValue(BUTTON_OFF);

				contact7->SetValue(BUTTON_OFF);
			
			
			} else /* flops = 0 */ {
				symVert->SetValue(BUTTON_ON );
				
				express1->SetValue(BUTTON_ON );
				express2->SetValue(BUTTON_OFF);
				express3->SetValue(BUTTON_OFF);
				express7->SetValue(BUTTON_OFF);

				contact7->SetValue(BUTTON_OFF);
			}
			break;
			
		
		case class3A:
			if (itsRots == 2) {
			
				if (itsFlops == 1) {
					symVert->SetValue(BUTTON_OFF);
					
					express1->SetValue(BUTTON_OFF);
					express2->SetValue(BUTTON_ON );
					express3->SetValue(BUTTON_OFF);
					express7->SetValue(BUTTON_OFF);
	
					contact7->SetValue(BUTTON_OFF);
				
				
				} else /* flops = 0 */ {
					symVert->SetValue(BUTTON_ON );
					
					express1->SetValue(BUTTON_OFF);
					express2->SetValue(BUTTON_ON );
					express3->SetValue(BUTTON_OFF);
					express7->SetValue(BUTTON_OFF);
	
					contact7->SetValue(BUTTON_OFF);
				}


			} else /* contacts = 7 */ {
				symVert->SetValue(BUTTON_ON );
				
				express1->SetValue(BUTTON_OFF);
				express2->SetValue(BUTTON_OFF);
				express3->SetValue(BUTTON_OFF);
				express7->SetValue(BUTTON_OFF);

				contact7->SetValue(BUTTON_ON );
			}
			break;
	
			
		case class4:
			if (itsFlops == 1) {
				symVert->SetValue(BUTTON_OFF);
				
				express1->SetValue(BUTTON_OFF);
				express2->SetValue(BUTTON_OFF);
				express3->SetValue(BUTTON_ON );
				express7->SetValue(BUTTON_OFF);

				contact7->SetValue(BUTTON_OFF);
			
			
			} else /* flops = 0 */ {
				symVert->SetValue(BUTTON_ON );
				
				express1->SetValue(BUTTON_OFF);
				express2->SetValue(BUTTON_OFF);
				express3->SetValue(BUTTON_ON );
				express7->SetValue(BUTTON_OFF);

				contact7->SetValue(BUTTON_OFF);
			}
			break;
			
			
		case class5:
			symVert->SetValue(BUTTON_ON );
			
			express1->SetValue(BUTTON_OFF);
			express2->SetValue(BUTTON_OFF);
			express3->SetValue(BUTTON_OFF);
			express7->SetValue(BUTTON_ON );

			contact7->SetValue(BUTTON_OFF);
			
			break;
			
			
		default:
			symVert->SetValue(BUTTON_OFF);
			
			express1->SetValue(BUTTON_OFF);
			express2->SetValue(BUTTON_OFF);
			express3->SetValue(BUTTON_OFF);
			express7->SetValue(BUTTON_OFF);

			contact7->SetValue(BUTTON_OFF);
			
			break;
	}
}	
	
	

/***
 * CheckBoxesToAttrs (override)
 *
 *	Encode checkbox settings into attribute values
 *
 ***/
 
void CFaceDialog::CheckBoxesToAttrs (void)

{
	if (contact7->IsChecked()) {
		itsClass = class3A;
		itsRots  = 7;
		itsFlops = 0;
		
		
	} else if (express1->IsChecked()) {
	
		if (symVert->IsChecked()) {
			itsClass = class2;
			itsRots  = 1;
			itsFlops = 0;
			
		} else {
			itsClass = class2;
			itsRots  = 1;
			itsFlops = 1;
		}	
		
	} else if (express2->IsChecked()) {
	
		if (symVert->IsChecked()) {
			itsClass = class3A;
			itsRots  = 2;
			itsFlops = 0;
			
		} else {
			itsClass = class3A;
			itsRots  = 2;
			itsFlops = 1;	
		}	
			
	} else if (express3->IsChecked()) {
	
		if (symVert->IsChecked()) {
			itsClass = class4;
			itsRots  = 3;
			itsFlops = 0;
			
		} else {
			itsClass = class4;
			itsRots  = 3;
			itsFlops = 1;	
		}
			
	} else	if (express7->IsChecked()) {
		itsClass = class5;
		itsRots  = 7;
		itsFlops = 0;
			
			
	} else /* default */ {
	 
	 	if (symVert->IsChecked()) {
			itsClass = class1;
			itsRots  = 0;
			itsFlops = 0;
			
		} else {
			itsClass = class1;
			itsRots  = 0;
			itsFlops = 1;	
		}
	}
}	




/**************************  Shoulder dialog methods  *************************/


/***
 * ISholDialog
 *
 *  Initialize a new instance of a hand attributes dialog.
 *
 ***/

void CSholDialog::ISholDialog (CDirectorOwner *aSupervisor)

{	
	Str27	winTitle = "\pShoulder Symbol Attributes";


	// do inherited
	
	CSymWndDialog::ISymWndDialog(DLOGsholAttrs, aSupervisor);
			     
    
	// Translate dialog resource item numbers into checkbox objects
	
	symVert = (CCheckBox*) itsWindow->FindViewByID (DITLsholSymVert);
	
	shol1 = (CCheckBox*) itsWindow->FindViewByID (DITLsholRot1);
	shol2 = (CCheckBox*) itsWindow->FindViewByID (DITLsholRot2);
	

	// Assign local command numbers to checkbox objects

	symVert->SetClickCmd (DITLsholSymVert);
	
	shol1->SetClickCmd (DITLsholRot1);
	shol2->SetClickCmd (DITLsholRot2);
	
	
	// Set window title
	
	itsWindow->SetTitle(winTitle);
	
}
	
	
		
/***
 * DoCommand (override)
 *
 *  Maintain checkbox consistency during dialog.
 *
 ***/

void CSholDialog::DoCommand (long theCommand)

{
	switch (theCommand) {
			
		case DITLsholSymVert:
			break;
			
		
		case DITLsholRot1: 
			if (shol1->IsChecked()) {
				shol2->SetValue(BUTTON_OFF);
			}
			break;
			
		
		case DITLsholRot2: 
			if (shol2->IsChecked()) {
				shol1->SetValue(BUTTON_OFF);
			}	
			break;
		
		
		default:
			inherited::DoCommand(theCommand);
			break;
	}
}



/***
 * AttrsToCheckBoxes (override)
 *
 *  Decode attribute values into checkbox settings
 *	
 ***/
 
void CSholDialog::AttrsToCheckBoxes (void)

{
	switch (itsClass) {
	
		case class1:
			if (itsRots == 1) {
			
				if (itsFlops == 1) {
					symVert->SetValue(BUTTON_OFF);
					
					shol1->SetValue(BUTTON_ON );
					shol2->SetValue(BUTTON_OFF);		
				
				} else /* flops = 0 */ {
					symVert->SetValue(BUTTON_ON );
					
					shol1->SetValue(BUTTON_ON );
					shol2->SetValue(BUTTON_OFF);	
				}

			} else /* rots = 0 */ {
				
				if (itsFlops == 1) {
					symVert->SetValue(BUTTON_OFF);
					
					shol1->SetValue(BUTTON_OFF);
					shol2->SetValue(BUTTON_OFF);		
				
				} else /* flops = 0 */ {
					symVert->SetValue(BUTTON_ON );
					
					shol1->SetValue(BUTTON_OFF);
					shol2->SetValue(BUTTON_OFF);	
				}		
			}
			break;
			
			
		case class2:
			if (itsFlops == 1) {
				symVert->SetValue(BUTTON_OFF);
				
				shol1->SetValue(BUTTON_OFF);
				shol2->SetValue(BUTTON_ON );	
			
			} else /* flops = 0 */ {
				symVert->SetValue(BUTTON_ON );
				
				shol1->SetValue(BUTTON_OFF);
				shol2->SetValue(BUTTON_ON );				
			}
			break;
	
	
		default:
			symVert->SetValue(BUTTON_OFF);
			
			shol1->SetValue(BUTTON_OFF);
			shol2->SetValue(BUTTON_OFF);
		
			break;
	}
}	
	
	

/***
 * CheckBoxesToAttrs (override)
 *
 *	Encode checkbox settings into attribute values
 *
 ***/
 
void CSholDialog::CheckBoxesToAttrs (void)

{
	if (shol1->IsChecked()) {
	
		if (symVert->IsChecked()) {
			itsClass = class1;
			itsRots  = 1;
			itsFlops = 0;
			
		} else {
			itsClass = class1;
			itsRots  = 1;
			itsFlops = 1;
		}	
		
	} else if (shol2->IsChecked()) {
	
		if (symVert->IsChecked()) {
			itsClass = class2;
			itsRots  = 2;
			itsFlops = 0;
			
		} else {
			itsClass = class2;
			itsRots  = 2;
			itsFlops = 1;	
		}	
			
			
	} else /* default */ {
	 
	 	if (symVert->IsChecked()) {
			itsClass = class1;
			itsRots  = 0;
			itsFlops = 0;
			
		} else {
			itsClass = class1;
			itsRots  = 0;
			itsFlops = 1;	
		}
	}
}		



/**************************  Contact dialog methods  ****************************/


/***
 * IContDialog
 *
 *  Initialize a new instance of a hand attributes dialog.
 *
 ***/

void CContDialog::IContDialog (CDirectorOwner *aSupervisor)

{	
	Str27	winTitle = "\pContact Symbol Attributes";


	// do inherited
	
	CSymWndDialog::ISymWndDialog(DLOGcontAttrs, aSupervisor);
			     
    
	// Translate dialog resource item numbers into checkbox objects
	
	symVert = (CCheckBox*) itsWindow->FindViewByID (DITLcontSymVert);
	

	// Assign local command numbers to checkbox objects

	symVert->SetClickCmd (DITLcontSymVert);
	
	
	// Set window title
	
	itsWindow->SetTitle(winTitle);
	
}
	
	
		
/***
 * DoCommand (override)
 *
 *  Maintain checkbox consistency during dialog.
 *
 ***/

void CContDialog::DoCommand (long theCommand)

{
	switch (theCommand) {
			
		case DITLcontSymVert:
			break;
		
		
		default:
			inherited::DoCommand(theCommand);
			break;
	}
}



/***
 * AttrsToCheckBoxes (override)
 *
 *  Decode attribute values into checkbox settings
 *	
 ***/
 
void CContDialog::AttrsToCheckBoxes (void)

{
	switch (itsClass) {
	
		case class1:
			if (itsFlops == 1) {
				symVert->SetValue(BUTTON_OFF);
			
			} else /* flops = 0 */ {
				symVert->SetValue(BUTTON_ON );
								
			}
			break;
	
	
		default:
			symVert->SetValue(BUTTON_OFF);
			
			break;
	}
}	
	
	

/***
 * CheckBoxesToAttrs (override)
 *
 *	Encode checkbox settings into attribute values
 *
 ***/
 
void CContDialog::CheckBoxesToAttrs (void)

{
 	if (symVert->IsChecked()) {
		itsClass = class1;
		itsRots  = 0;
		itsFlops = 0;
		
	} else {
		itsClass = class1;
		itsRots  = 0;
		itsFlops = 1;	
	}
}		



/**************************  'Other' dialog methods  ****************************/


/***
 * IOthrDialog
 *
 *  Initialize a new instance of a hand attributes dialog.
 *
 ***/

void COthrDialog::IOthrDialog (CDirectorOwner *aSupervisor)

{	
	Str27	winTitle = "\pOther Symbol Attributes";


	// do inherited
	
	CSymWndDialog::ISymWndDialog(DLOGothrAttrs, aSupervisor);
			     
    
	// Translate dialog resource item numbers into checkbox objects
	
	symVert = (CCheckBox*) itsWindow->FindViewByID (DITLothrSymVert);
	
	line2 = (CCheckBox*) itsWindow->FindViewByID (DITLothrLine2);
	line3 = (CCheckBox*) itsWindow->FindViewByID (DITLothrLine3);
	

	// Assign local command numbers to checkbox objects

	symVert->SetClickCmd (DITLothrSymVert);
	
	line2->SetClickCmd (DITLothrLine2);
	line3->SetClickCmd (DITLothrLine3);
	
	
	// Set window title
	
	itsWindow->SetTitle(winTitle);
	
}
	
	
		
/***
 * DoCommand (override)
 *
 *  Maintain checkbox consistency during dialog.
 *
 ***/

void COthrDialog::DoCommand (long theCommand)

{
	switch (theCommand) {
			
		case DITLothrSymVert:
			break;
			
		
		case DITLothrLine2: 
			if (line2->IsChecked()) {
				line3->SetValue(BUTTON_OFF);
			}
			break;
			
		
		case DITLothrLine3: 
			if (line3->IsChecked()) {
				line2->SetValue(BUTTON_OFF);
			}	
			break;
		
		
		default:
			inherited::DoCommand(theCommand);
			break;
	}
}



/***
 * AttrsToCheckBoxes (override)
 *
 *  Decode attribute values into checkbox settings
 *	
 ***/
 
void COthrDialog::AttrsToCheckBoxes (void)

{
	switch (itsClass) {
	
		case class1:
			if (itsFlops == 1) {
				symVert->SetValue(BUTTON_OFF);
				
				line2->SetValue(BUTTON_OFF);
				line3->SetValue(BUTTON_OFF);	
			
			} else /* flops = 0 */ {
				symVert->SetValue(BUTTON_ON );
				
				line2->SetValue(BUTTON_OFF);
				line3->SetValue(BUTTON_OFF);				
			}
			break;
	
	
		case class3A:
			if (itsRots == 2) {
			
				if (itsFlops == 1) {
					symVert->SetValue(BUTTON_OFF);
					
					line2->SetValue(BUTTON_ON );
					line3->SetValue(BUTTON_OFF);		
				
				} else /* flops = 0 */ {
					symVert->SetValue(BUTTON_ON );
					
					line2->SetValue(BUTTON_ON );
					line3->SetValue(BUTTON_OFF);	
				}

			} else /* rots = 3 */ {
				
				if (itsFlops == 1) {
					symVert->SetValue(BUTTON_OFF);
					
					line2->SetValue(BUTTON_OFF);
					line3->SetValue(BUTTON_ON );		
				
				} else /* flops = 0 */ {
					symVert->SetValue(BUTTON_ON );
					
					line2->SetValue(BUTTON_OFF);
					line3->SetValue(BUTTON_ON );	
				}
				
			}
			break;
	
		default:
			symVert->SetValue(BUTTON_OFF);
			
			line2->SetValue(BUTTON_OFF);
			line3->SetValue(BUTTON_OFF);
		
			break;
	}
}	
	
	

/***
 * CheckBoxesToAttrs (override)
 *
 *	Encode checkbox settings into attribute values
 *
 ***/
 
void COthrDialog::CheckBoxesToAttrs (void)

{
	if (line2->IsChecked()) {
	
		if (symVert->IsChecked()) {
			itsClass = class3A;
			itsRots  = 2;
			itsFlops = 0;
			
		} else {
			itsClass = class3A;
			itsRots  = 2;
			itsFlops = 1;
		}	
		
	} else if (line3->IsChecked()) {
	
		if (symVert->IsChecked()) {
			itsClass = class3A;
			itsRots  = 3;
			itsFlops = 0;
			
		} else {
			itsClass = class3A;
			itsRots  = 3;
			itsFlops = 1;	
		}	
			
			
	} else /* default */ {
	 
	 	if (symVert->IsChecked()) {
			itsClass = class1;
			itsRots  = 0;
			itsFlops = 0;
			
		} else {
			itsClass = class1;
			itsRots  = 0;
			itsFlops = 1;	
		}
	}
}		