/****
 * CSymWndGrid.c
 *
 *	Symbol window methods for the SignWriter symbol editor.
 *
 *  Copyright 1995 The Center for Sutton Movement Writing. All rights reserved.
 *
 ****/

 
/**
 *
 *  This class subclasses TCL's CGridSelector class to implement
 *  the symbol editor's symbol window.
 *
 **/


#include "CSymWndGrid.h"
#include "CBureaucrat.h"
#include "CFileWndGrid.h"
#include "CImgWindow.h"
#include "CImgWndPane.h"
#include "CSelector.h"
#include "CSymWndBox.h"
#include "CSymWndTask.h"
#include "CWindow.h"
#include "LongCoordinates.h"
#include "LongQD.h"
#include "SymEditConsts.h"
#include "Symbols.h"
#include "TBUtilities.h"





/*** Global Variables ***/

extern void			*gClipRec;			/* Clipboard */
extern short	 	 gClipItem;

extern CFileWndGrid	*gFileWind;			/* File window grid */





/****************************  Initialization  ****************************/


/***
 * ISymWndGrid
 *
 ***/
 
void CSymWndGrid::ISymWndGrid (CView *anEnclosure, CBureaucrat *aSupervisor,
							   short aWidth, short aHeight,
							   short aHEncl, short aVEncl,
							   SizingOption aHSizing, SizingOption aVSizing,
							   short symCode)
{
	short var, row, col;
	
	itsSymCode = symCode;
	itsSymVar  = 1;
	
	itsImgWindows = 0;
	
	for (var = 0; var < SGRID_VARS; var++)
		for (row = 0; row < SGRID_COLS; row++)
			for (col = 0; col < SGRID_ROWS; col++)
				itsImgWindow[var][col][row] = NULL;
				
	inherited::IGridSelector(anEnclosure, aSupervisor,
							aHEncl, aVEncl, aHSizing, aVSizing,
							SELnone, 0, SGRID_ROWS, SGRID_COLS,
							SCELL_WIDTH, SCELL_HEIGHT);
}




 /****************************  Window management  ****************************/


/***
 * GetImageWindow
 *
 *	Return the window corresponding
 *  to the specified symbol image.
 *
 ***/
 
void CSymWndGrid::GetImageWindow (short symVar, short fillVar, short image, void **wind)
{
	*wind = itsImgWindow[symVar-1][fillVar-1][image-1];
}



/***
 * SetImageWindow
 *
 *	Assign a window to the 
 *  specified symbol image.
 *
 ***/
 
void CSymWndGrid::SetImageWindow (short symVar, short fillVar, short image, void *wind)
{
	itsImgWindow[symVar-1][fillVar-1][image-1] = wind;
	
	if (wind != NULL)
		itsImgWindows++;
	else 
		itsImgWindows--;
}



/***
 * ImageWindowsOpen
 *
 *	Check for open sym windows.
 *
 ***/
 
Boolean CSymWndGrid::ImageWindowsOpen()

{
	short junk;
	
	if (itsImgWindows > 0) {
	
		PositionDialog('ALRT', ALRTopenImgs);	
		junk = Alert(ALRTopenImgs, NULL);	
		
		return TRUE;
	}
	
	return FALSE;	
}




 /******************************  Grid mapping  *******************************/


/***
 * ItemToSymImage
 *
 *	Convert a grid class item number
 *  to the corresponding 1-based fill 
 *  variation (column) and image (row).
 *
 ***/
 
void CSymWndGrid::ItemToSymImage (short item, short *fillVar, short *image)
{
	item--;
 	*fillVar = 1 + (item % SGRID_COLS);
	*image  = 1 + (item / SGRID_COLS);
}
 
 
 
/***
 * SymImageToItem
 *
 *	Convert a symbol image to the
 *  corresponding grid class item.
 *
 ***/
 
void CSymWndGrid::SymImageToItem (short fillVar, short image, short *item)
{
 	*item = ((image - 1) * SGRID_COLS) + fillVar;
}


 
/***
 * ValidItem
 *
 *  Return TRUE if the specified item
 *  is a defined symbol cell.
 *
 ***/
 
Boolean CSymWndGrid::ValidItem (short theItem) 
{
	ImageClass 	imgClass;
	short 	 	fill, maxFill;
	short 	 	image, maxImage;
	
	if (theItem <= 0)
		return FALSE;	
				   		
 	ItemToSymImage(theItem, &fill, &image);	
 	GetFillVars(itsSymCode, itsSymVar, &maxFill);
 	
	if (fill > maxFill)
		return FALSE;			   		

	GetClass(itsSymCode, itsSymVar, fill, &imgClass);
	GetImages(imgClass, &maxImage);

 	return image <= maxImage;
}
 	
 	
 	
 	
/********************************  Drawing  ********************************/


/***
 * Update
 *
 *  Update the symbol window grid to display
 *  the specifed symbol variation.
 *
 ***/
 
void CSymWndGrid::Update (short symVar)
{
	LongRect	theFrame;
	Rect		theRect;
		
	itsSymVar = symVar;
	
	// Deselect
	Prepare();
	CSelector::ChangeSelection(SELnone);

	// Redraw
	GetFrame(&theFrame);
	FrameToQDR(&theFrame, &theRect);
	inherited::Draw(&theRect);
	
	// Update file window
	gFileWind->UpdateItem(itsSymCode);
	Prepare();
	
}



/***
 * DrawItem
 *
 *  Draw the specified grid item.  The item numbering
 *  scheme is as defined for CGridSelector.
 *
 ***/
 
void CSymWndGrid::DrawItem (short theItem, Rect *theBox)
 
{ 
	short		fill;
	short		image;
	CImgWindow  *imgWind;
	Rect		bounds;
	LongPt		offset;
	LongPt		junk;


 	if (!ValidItem(theItem)) {
 		FillRect(theBox, ltGray);
 		return;
 	}
 	 			
	ItemToSymImage(theItem, &fill, &image);
	GetImageWindow(itsSymVar, fill, image, (void **) &imgWind);

	if (imgWind != NULL) {		// get current image from window
	
		SetRect(&bounds, 0, 0, IMAP_WIDTH, IMAP_HEIGHT);
		CopyBits(imgWind->itsEditPane->itsBitMap->macBitMap, &thePort->portBits, 
				 &bounds, theBox, srcCopy, NULL);
				  
	} else {					// get image from data base
	 
	 	FillRect(theBox, white);
	 	
	 	offset.h = theBox->left;
	 	offset.v = theBox->top;
	 	
		DrawImage(itsSymCode, itsSymVar, fill, image, &thePort->portBits, &offset);
	} 	
}

	
 
/***
 * UpdateItem
 *
 *  Update the specified grid
 *  item during image editing.
 *
 ***/

void CSymWndGrid::UpdateItem (short symVar, short fillVar, short image, 
				 			  BitMap *img, LongRect *area)
{
	short		item;
	Rect		bounds;
	LongRect	dest;
	RgnHandle	save;
	
 	if (symVar == itsSymVar) {
 	
 		Prepare();

 		save = NewRgn();
 		GetClip(save);
 		
	 	SymImageToItem(fillVar, image, &item);
	 	FindItemBox(item, &bounds);
	 	
	 	dest = *area;
	 	OffsetLongRect(&dest, bounds.left, bounds.top);
	 	
		ClipRect(&bounds);
		LCopyBits(img, &thePort->portBits, area, &dest, srcCopy, NULL);
		
		SetClip(save);
		DisposeRgn(save);
		
	}
}
 
 
 
 
/********************************  Selection  ********************************/


/***
 * HiliteItem (override default)
 *
 *  Highlight the specified grid item.  If a grid column
 *  or the entire grid is currently selected, highlight
 *  the corresponding items.
 *
 ***/
 
void CSymWndGrid::HiliteItem (short theItem, HiliteState state)

{
	short item, col, select, junk;	
	
	if (theItem > 0)
		inherited::HiliteItem(theItem, state);

	else {
	
		select = -theItem;
		for (item = 1; item <= (rows * columns); item++) {
		
			ItemToSymImage(item, &col, &junk);
			if (ValidItem(item) && ((theItem == SELall) || (col == select)))
				inherited::HiliteItem(item, state);
		}
	}
}



/***
 * SelectColumn
 *
 *  Select the specified column. Column
 *  selection is marked by a negative
 *  item number indicating the column.
 *
 ***/
 
void CSymWndGrid::SelectColumn (short col)

{
	short fills;
	
	Prepare();
	GetFillVars(itsSymCode, itsSymVar, &fills);
	
	if (col <= fills)
		CSelector::ChangeSelection(-col);
}



/***
 * SelectAll
 *
 *  Select all items in the symbol window grid.
 *
 ***/
 
void CSymWndGrid::SelectAll (void)

{	
	Prepare();
	CSelector::ChangeSelection(SELall);
}




/********************************  Clipboard  ********************************/


/***
 * GetRec
 *
 *  Return symbol record corresponding to the specified grid item.
 *
 ***/
 
void CSymWndGrid::GetRec (short item, void **rec)

{
	short fill, image;
	
	if (item == SELall) {
		GetSymVarRec(itsSymCode, itsSymVar, rec);
	
	} else if (item < 0) {
		ItemToSymImage(-item, &fill, &image);
		GetFillVarRec(itsSymCode, itsSymVar, fill, rec);		

	} else {
		ItemToSymImage(item, &fill, &image);
		GetImageRec(itsSymCode, itsSymVar, fill, image, rec);
	}		  	
}
				  


/***
 * SetRec
 *
 *  Update symbol record corresponding to the specified grid item.
 *
 ***/
 
void CSymWndGrid::SetRec (short item, void *rec)

{
	short fill, image;
	
	if (item == SELall) {
		SetSymVarRec(itsSymCode, itsSymVar, rec);
	
	} else if (item < 0) {
		ItemToSymImage(-item, &fill, &image);
		SetFillVarRec(itsSymCode, itsSymVar, fill, rec);		

	} else {
		ItemToSymImage(item, &fill, &image);
		SetImageRec(itsSymCode, itsSymVar, fill, image, rec);
	}		  	
}
					  

		
/***
 * CopyRec
 *
 *  Copy the specified symbol record.
 *
 ***/
 
void CSymWndGrid::CopyRec (short item, void *rec, void **copyRec)

{
	if (item == SELall)
		CopySymVarRec(rec, copyRec);
	
	else if (item < 0)
		CopyFillVarRec(rec, copyRec);		

	else
		CopyImageRec(rec, copyRec);		  	
}
					  

		
/***
 * ClearRec
 *
 *  Clear the specified symbol record.
 *
 ***/
 
void CSymWndGrid::ClearRec (short item, void **rec)

{
	if (item == SELall)
		ClearSymVarRec(rec);
	
	else if (item < 0)
		ClearFillVarRec(rec);		

	else
		ClearImageRec(rec);		  	
}


				 	
/***
 * DisposRec
 *
 *  Dispose the specified symbol record.
 *
 ***/
 	
void CSymWndGrid::DisposRec (short item, void *rec)

{
	if (item == SELall)
		DisposSymVarRec(rec);
	
	else if (item < 0)
		DisposFillVarRec(rec);		

	else
		DisposImageRec(rec);		  	
}




/********************************  Commands  *******************************/


	
/***
/* 
/*  DoCopy
/*
/*  Copy selection to the clipboard. Not undoable.
/*
 ***/
		
void CSymWndGrid::DoCopy()

{
	
	short	select = CSelector::GetSelection();
	void	*rec;
		
	if (gClipRec != NULL)
		DisposRec(gClipItem, gClipRec);
	
	GetRec(select, &rec);
	CopyRec(select, rec, &gClipRec);
	
	gClipItem = select;
}
	


/***
/* 
/*  DoClear
/*
/*  Erase the current selection
/*
 ***/

void CSymWndGrid::DoClear (short taskIndex)

{
	short		   select = CSelector::GetSelection();
	CSymWndClear  *theClearTask;
	
	if (ImageWindowsOpen())
		return;
											
	theClearTask = new CSymWndClear;
	theClearTask->ISymWndClear(taskIndex, this, select);
	theClearTask->Do();
	
	itsSupervisor->Notify(theClearTask);
	itsLastTask = theClearTask;
}



/***
/* 
/*  DoPaste
/*
/*  Paste sym record from the clipboard.
/*
 ***/

void CSymWndGrid::DoPaste()

{
	short		   select = CSelector::GetSelection();
	CSymWndPaste  *thePasteTask;
	
	if (ImageWindowsOpen())
		return;
											
	thePasteTask = new CSymWndPaste;
	thePasteTask->ISymWndPaste(this, select);
	thePasteTask->Do();
	
	itsSupervisor->Notify(thePasteTask);
	itsLastTask = thePasteTask;
}	




/**********************************  Mouse  ***********************************/


/***
 * DoClick (override default method)
 *
 *	Highlight the selected cell (unhighlighting the  
 *  previous one). 
 *
 ***/ 

void CSymWndGrid::DoClick (Point hitPt, short modifierKeys, long when)

{
	short item = FindItem(hitPt);
	
	if (!ValidItem(item))
		return;
		
	inherited::DoClick(hitPt, modifierKeys, when);
}



/***
 * DoDoubleClick (override default)
 *
 * 	Open the selected symbol image for editing.
 *
 ***/
 
 void CSymWndGrid::DoDoubleClick(void)
 
{
	short 		item;
	short 		fill;
	short 		image;
	CImgWindow  *imgWind;
	
	item = CSelector::GetSelection();
	if (!ValidItem(item))
		return;
		
	CSelector::ChangeSelection(SELnone);
	
	// window already open?
	ItemToSymImage(item, &fill, &image);
	GetImageWindow(itsSymVar, fill, image, (void *) &imgWind);
	
	if (imgWind != NULL)
	  imgWind->itsWindow->Select();
	else		
	  NewImageWindow(fill, image);
}




/******************************  Image editing  *******************************/


/***
 * NewImageWindow
 *
 *	Create new instance of an image window
 *	and send it a NewWindow message.
 *
 ***/

void CSymWndGrid::NewImageWindow(short fillVar, short image)
{
	CImgWindow	*theWindow = NULL;

	TRY
	{
		theWindow = new CImgWindow;
		
		theWindow->IImgWindow((CDirectorOwner*) this->itsSupervisor);
	
		theWindow->NewWindow(itsSymCode, itsSymVar, fillVar, image, this);
	}
	
	CATCH
	{
		 if (theWindow) theWindow->Dispose();
	}
	ENDTRY;
}


