/******************************************************************************
 CRowColSizer.h
 
 	A subclass of CMouseTask that handles resizing of rows and columns in
 	CDemoTable.
		
	SUPERCLASS = CMouseTask
	
	Copyright  1991 Symantec Corporation. All rights reserved.
	

 ******************************************************************************/

#include "CRowColSizer.h"
#include "Global.h"

#define		kMinNewSize		5

/******************************************************************************
 IRowColSizer
 
 	Initialization method for CRowColSizer
******************************************************************************/

void CRowColSizer::IRowColSizer( CTable *aTable, CTableLabels *aLabel, short  aRowOrCol, 
						VHSelect aDimension, LongRect *aLimitRect, short aTaskIndex)
{
	itsTable = aTable;
	itsLabel = aLabel;
	limitRect = *aLimitRect;
	whichDimension = aDimension;
	rowOrCol = aRowOrCol;
	
	CMouseTask::IMouseTask( aTaskIndex);
}

/******************************************************************************
 Dispose
  	
  	Dispose of a CRowColSizer. This method clears the table's and/or
  	label's reference to this task.
******************************************************************************/

void CRowColSizer::Dispose(void)
{
	if (itsTable->itsLastTask == this)
		itsTable->itsLastTask = NULL;
		
	if (itsLabel->itsLastTask == this)
		itsLabel->itsLastTask = NULL;
		
	inherited::Dispose();
}

/******************************************************************************
 BeginTracking {OVERRIDE}
 
 	Called at the start of dragging. Draws the initial feedback.
******************************************************************************/

void CRowColSizer::BeginTracking( LongPt *startPt)
{
	DrawFeedback( startPt);
}

/******************************************************************************
 KeepTracking {OVERRIDE}
 
 	Called continously during dragging. This method erases the old feedback,
 	and draws the new feedback.
******************************************************************************/

void CRowColSizer::KeepTracking( LongPt *currPt, LongPt *prevPt, LongPt *startPt)
{
	if (!EqualLongPt( currPt, prevPt))
	{
		DrawFeedback( prevPt);
		DrawFeedback( currPt);
	}
}

/******************************************************************************
 EndTracking {OVERRIDE}
 
 	Called at the end of dragging. This method erases the feedback, and 
 	resizes the row or column.
 	
******************************************************************************/

void CRowColSizer::EndTracking( LongPt *currPt, LongPt *prevPt, LongPt *startPt)
{
	long	start;
	short 	size;
	
	DrawFeedback( prevPt);
	PenNormal();
	
	if (whichDimension == h)
	{
		start = itsTable->GetColStart( rowOrCol);
		prevSize = itsTable->GetColWidth( rowOrCol);
		size = Abs(currPt->h - start);
		if (size > kMinNewSize)
		{
			itsTable->SetColWidth( rowOrCol, size);
			itsLabel->SetColWidth( rowOrCol, size);
		}
	}
	else
	{
		start = itsTable->GetRowStart( rowOrCol);
		prevSize = itsTable->GetRowHeight( rowOrCol);
		size = Abs(currPt->v - start);
		if (size > kMinNewSize)
		{
			itsTable->SetRowHeight( rowOrCol, size);
			itsLabel->SetRowHeight( rowOrCol, size);
		}	
	}
}

/******************************************************************************
 DrawFeedback
 
 	Called to draw visual feedback as the user drags. This method draws
 	a line to indicate the new size of the row or column.
 	
******************************************************************************/

void CRowColSizer::DrawFeedback( LongPt *currPt)
{
	short	dH, dV;
	LongPt	startPt;
	Point	qdPt;
	
	PenMode( patXor);
	
	if (whichDimension == h)	/* resizing a column	*/
	{
		SetLongPt( &startPt, currPt->h, limitRect.top);
		dH = 0;
		dV = limitRect.bottom - limitRect.top;
	}
	else						/* resizing a row		*/
	{
		SetLongPt( &startPt, limitRect.left, currPt->v);
		dH = limitRect.right - limitRect.left;
		dV = 0;	
	}
	itsTable->FrameToQD( &startPt, &qdPt);
	MoveTo( qdPt.h, qdPt.v);
	Line( dH, dV);
}

/******************************************************************************
 Undo {OVERRIDE}
 
 	Called to undo and redo a row or column resize.
******************************************************************************/

void CRowColSizer::Undo( void)
{
	short 	size = prevSize;

	itsTable->Prepare();
	if (whichDimension == h)
	{
		prevSize = itsTable->GetColWidth( rowOrCol);
		itsTable->SetColWidth( rowOrCol, size);
		itsLabel->SetColWidth( rowOrCol, size);
	}
	else
	{
		prevSize = itsTable->GetRowHeight( rowOrCol);
		itsTable->SetRowHeight( rowOrCol, size);	
		itsLabel->SetRowHeight( rowOrCol, size);	
	}
	itsLabel->Refresh();
}