/******************************************************************************
 CTextDoc.c

	A subclass of CDocument that handles text files, which may
	have a 'styl' resource for styled text.
		
	SUPERCLASS = CDocument
	
	Copyright  1991 Symantec Corporation. All rights reserved.
	

 ******************************************************************************/

#include <Global.h>
#include <Commands.h>
#include <CApplication.h>
#include <CBartender.h>
#include <CDataFile.h>
#include <CDecorator.h>
#include <CDesktop.h>
#include <CError.h>
#include <CPanorama.h>
#include <CScrollPane.h>
#include <TBUtilities.h>
#include "CTextDoc.h"
#include "CPrinter.h"
#include "CStyleText.h"
#include "CResFile.h"
#include "Constants.h"
#include "CWindow.h"
#include "Demo.h"

#define	kTextWindow		1024	/* Resource ID for WIND template */
#define kStyleScrapID	128

extern	CApplication *gApplication;	/* The application */
extern	CBartender	*gBartender;	/* The menu handling object */
extern	CDecorator	*gDecorator;	/* Window dressing object	*/
extern	CDesktop	*gDesktop;		/* The enclosure for all windows */
extern	CBureaucrat	*gGopher;		/* The current boss in the chain of command */
extern	OSType		gSignature;		/* The application's signature */
extern	CError		*gError;		/* The error handling object */

/******************************************************************************
 ITextDoc
 
 	Initialization method for CTextDoc. wantsStyles specifies whether styled
 	text is preferred.
 ******************************************************************************/

void CTextDoc::ITextDoc(CApplication *aSupervisor, Boolean printable, 
					Boolean wantsStyles)

{
	itsResFork = NULL;

	CDocument::IDocument(aSupervisor, printable);
	isStyledText = wantsStyles;
}

/******************************************************************************
 NewFile {OVERRIDE}
 
	 When the user chooses New from the File menu, the CreateDocument()
	 method in your Application class will send a newly created document
	 this message. This method needs to create a new window, ready to
	 work on a new document.
	 
	 Since this method and the OpenFile() method share the code for creating
	 the window, you should use an auxiliary window-building method.
 ******************************************************************************/

void CTextDoc::NewFile(void)

{
		/**
		 **	BuildWindow() is the method that
		 **	does the work of creating a window.
		 **	It's parameter should be the data that
		 **	you want to display in the window.
		 **	Since this is a new window, there's nothing
		 **	to display.
		 **
		 **/

	BuildWindow();

		/**
		 **	Send the window a Select() message to make
		 **	it the active window.
		 **/
	
	itsWindow->Select();
}

/******************************************************************************
 OpenFile {OVERRIDE}
 
	When the user chooses Open from the File menu, the OpenDocument()
	method in your Application class will let the user choose a file
 	and then send a newly created document this message. The information
	about the file is in the SFReply record.
	
	In this method, you need to open the file and display its contents
	in a window. This method use separate methods to create the window
	and install the data. 
 
 ******************************************************************************/


void CTextDoc::OpenFile(SFReply *macSFReply)

{
	CDataFile	*theFile;
	Handle		theText = NULL;
	StScrpHandle theStyles = NULL;
	Str63		theName;
	OSErr		theError;
	long		fileSize;

	TRY
	{
	
			// Create a new data file, and point it to
			// the file the user just selected.
			
		theFile = new(CDataFile);	
		itsFile = theFile;
	
		theFile->IDataFile();
		theFile->SFSpecify(macSFReply);
		
		theFile->Open( fsRdWrPerm);
		
			// Create a resource file object to handle styles
			
		itsResFork = new( CResFile);
		itsResFork->IResFile();
		itsResFork->SFSpecify( macSFReply);
			 
			// Read the data from the file. If this fails,
			// an exception will be raised
			
		ReadData( &theText, &theStyles);
						
			// Create the window. It will contain no data
			
		BuildWindow();
		
			// Install the data from the file into the window
			
		InstallData( theText, theStyles);
		
			// Free the data, since InstallData makes a copy
			
		ForgetHandle(theText);
		ForgetHandle( theStyles);
		
			// Set the window title to the file name and
			// show the window
			
		itsFile->GetName(theName);
		itsWindow->SetTitle(theName);
		itsWindow->Select();
	}
	CATCH
	{
		ForgetHandle( theText);
		ForgetHandle( theStyles);
	}
	ENDTRY;
}

/******************************************************************************
 BuildWindow

	This is the auxiliary window-building method that the
	NewFile() and OpenFile() methods use to create a window.
 
 ******************************************************************************/

void CTextDoc::BuildWindow(void)

{
	CScrollPane		*theScrollPane;
	CStyleText		*styleText;
	CEditText		*plainText;
	Rect			margin, sizeRect;

		/**
		 **	First create the window and initialize
		 **	it. The first argument is the resource ID
		 **	of the window. The second argument specifies
		 **	whether the window is a floating window.
		 **	The third argument is the window's enclosure; it
		 **	should always be gDesktop. The last argument is
		 **	the window's supervisor in the Chain of Command;
		 **	it should always be a document or director object.
		 **
		 **/

	itsWindow = new(CWindow);
	itsWindow->IWindow( kTextWindow, FALSE, gDesktop, this);
	
		/**
		 **	After you create the window, you can use the
		 **	SetSizeRect() message to set the windows maximum
		 **	and minimum size. Be sure to set the max & min
		 **	BEFORE you send a PlaceNewWindow() message to the
		 **	decorator.
		 **
		 ** The default minimum is 100 by 100 pixels. The
		 **	default maximum is the bounds of GrayRgn() (The
		 **	entire display area on all screens.)
		 **
		 **/

	theScrollPane = new(CScrollPane);
	
		/**
		 **	You can initialize a scroll pane two ways:
		 **		1. You can specify all the values
		 **		   right in your code, like this.
		 **		2. You can create a ScPn resource and
		 **		   initialize the pane from the information
		 **		   in the resource.
		 **
		 **/

	theScrollPane->IScrollPane(itsWindow, this, 10, 10, 0, 0,
								sizELASTIC, sizELASTIC,
								TRUE, TRUE, TRUE);

		/**
		 **	The FitToEnclFrame() method makes the
		 **	scroll pane be as large as its enclosure.
		 **	In this case, the enclosure is the window,
		 **	so the scroll pane will take up the entire
		 **	window.
		 **
		 **/

	theScrollPane->FitToEnclFrame(TRUE, TRUE);


		/**
		 **	itsMainPane is the document's focus
		 **	of attention. Some of the standard
		 **	classes (particularly CPrinter) rely
		 **	on itsMainPane pointing to the main
		 **	pane of your window.
		 **
		 **	itsGopher specifies which object
		 **	should become the gopher. By default
		 **	the document becomes the gopher. Its
		 **	likely that your main pane handles commands
		 **	so youll almost want to set itsGopher
		 **	to point to the same object as itsMainPane.
		 **
		 **	Note that the main pane is the
		 **	panorama in the scroll pane and not
		 **	the scroll pane itself.
		 **
		 **/

	if (isStyledText)
	{
		styleText = new( CStyleText);
		styleText->IStyleText( theScrollPane, this, 0, 0, 0, 0,
				sizELASTIC, sizELASTIC, -1);
		itsMainPane = styleText;
	}
	else
	{
		plainText = new( CEditText);
		plainText->IEditText( theScrollPane, this, 0, 0, 0, 0,
				sizELASTIC, sizELASTIC, -1);
		itsMainPane = plainText;
	
	}
	itsGopher = itsMainPane;

	itsMainPane->FitToEnclosure( TRUE, TRUE);
	itsMainPane->Offset( 2, 2, FALSE);
	SetRect( &margin, 0, 0, -2, -2);
	itsMainPane->ChangeSize( &margin, FALSE);

		/**
		 **	Send the scroll pane an InstallPanorama()
		 **	to associate our pane with the scroll pane.
		 **
		 **/

	theScrollPane->InstallPanorama( (CPanorama*)itsMainPane);
			
	sizeRect = itsWindow->sizeRect;
	
	sizeRect.right = Min( pageWidth + SBARSIZE, sizeRect.right);
	sizeRect.bottom = Min( pageHeight + SBARSIZE, sizeRect.bottom);
	itsWindow->SetSizeRect( &sizeRect);
	
		/**
		 **	The Decorator is a global object that takes care
		 **	of placing and sizing windows on the screen.
		 **	You don't have to use it.
		 **
		 **/

	gDecorator->PlaceNewWindow(itsWindow);
}


/******************************************************************************
 DoSave {OVERRIDE}
 
 	This method handles what happens when the user chooses Save from the
 	File menu. This method should return TRUE if the file save was successful.
 	If there is no file associated with the document, you should send a
 	DoSaveFileAs() message.

 ******************************************************************************/

Boolean CTextDoc::DoSave(void)

{
	Handle			theText;
	StScrpHandle 	styles = NULL;
	long			selStart, selEnd;
	Boolean			addResSucceeded = FALSE;
	short			savedRes = -1;
	CStyleText		*styleText = (CStyleText*)itsMainPane;

	if (itsFile == NULL)
		return(DoSaveFileAs());
		
	TRY
	{
		theText = ((CEditText*)itsMainPane)->GetTextHandle();

		((CDataFile *)itsFile)->WriteAll(theText);
		
		if (isStyledText)
		{
			savedRes = CurResFile();
			
			if (!itsResFork->HasResFork())
				itsResFork->CreateNew( gSignature, 'TEXT');
				
			itsResFork->Open( fsRdWrPerm);
				
			itsResFork->MakeCurrent();
			
			styles = (StScrpHandle) Get1Resource( 'styl', kStyleScrapID);
			if (styles)
			{
				RmveResource( (Handle) styles);
				FailResError();
				ForgetHandle( styles);
			}
				
			styleText->GetSelection( &selStart, &selEnd);
			styleText->SetSelection( 0, MAXINT, FALSE);
				
			styles = styleText->GetStyleScrap();
			styleText->SetSelection( selStart, selEnd, FALSE);
					
			if (styles)
			{
				AddResource( (Handle) styles, 'styl', kStyleScrapID, "\p");
				FailResError();
				addResSucceeded = TRUE;
				itsResFork->Update();
				ForgetResource( styles);
			}
			
			UseResFile( savedRes);	
			savedRes = -1;
			
			itsResFork->Close();		
		}	
	}
	CATCH
	{
		if (addResSucceeded)
			ForgetResource( styles);
		else
			ForgetHandle( styles);
			
		if (savedRes != -1)
			UseResFile( savedRes);
	}
	ENDTRY;
	
	dirty = FALSE;		// Document is no longer dirty

	return TRUE;		// Save was successful
}


/******************************************************************************
 DoSaveAs {OVERRIDE}
 
	This method handles what happens when the user chooses Save As from
 	File menu. The default DoCommand() method for documents sends a DoSaveFileAs()
 	message which displays a standard put file dialog and sends this message.
 	The SFReply record contains all the information about the file you're about
 	to create.

 ******************************************************************************/

Boolean CTextDoc::DoSaveAs(SFReply *macSFReply)

{
		/**
		 **	If there's a file associated with this document
		 **	already, close it. The Dispose() method for files
		 **	sends a Close() message to the file before releasing
		 **	its memory.
		 **
		 **/
		 
	if (itsFile != NULL)
		itsFile->Dispose();


		/**
		 **	Create a new file, and then save it normally.
		 **
		 **/

	itsFile = new(CDataFile);
	((CDataFile *)itsFile)->IDataFile();
	itsFile->SFSpecify(macSFReply);
	
		/**
		 ** If this is a styled text document
		 ** make a resource fork for the file
		 **
		 **/
	
	if (isStyledText)
	{
		itsResFork = new( CResFile);
		itsResFork->IResFile();
		itsResFork->SFSpecify( macSFReply);
	}
	
		/**
		 ** Delete existing file if it exists
		 **
		 **/

	if (itsFile->ExistsOnDisk())
		itsFile->ThrowOut();
		
		/**
		 ** Create a new file,with our creator signature and of type 'TEXT'
		 **
		 **/
		 
	itsFile->CreateNew(gSignature, 'TEXT');
	itsFile->Open(fsRdWrPerm);
	
	itsWindow->SetTitle(macSFReply->fName);

	return( DoSave() );
}

/******************************************************************************
 ReadData
 
 	This method reads the text file and returns the text in the handle
 	theText. If there is a 'styl' resource, it is detached from the resource
 	file and returned in the handle theStyles.
 	
 ******************************************************************************/


void CTextDoc::ReadData( Handle *theText, StScrpHandle *theStyles)
{
	long 		fileSize;
	CDataFile 	*dataFile;
	
		// clear the handles in case of an error
		
	*theText = NULL;
	*theStyles = NULL;
	
		// We are assuming that we have data and resource file objects
		// already created
		
	ASSERT( itsFile && itsResFork);
	
	TRY
	{
			// Check that the file is not too big for TextEdit, and that
			// there is enough memory to read the file.
			
		dataFile = (CDataFile*)itsFile;
		fileSize = dataFile->GetLength();
		if ((fileSize > kMaxTELength) || (MaxBlock() < fileSize))
		{
			Failure( memFullErr,  SpecifyMsg( kDemoExceptionStrings, kFileTooBigErr));
		}
		         
		*theText = dataFile->ReadAll();
				
			// Read the style information if it is wanted and any exists
			
		if (isStyledText && itsResFork->HasResFork())
		{
			short savedRes = CurResFile();
			
			itsResFork->Open( fsRdWrPerm);
			itsResFork->MakeCurrent();
			
			*theStyles = (StScrpHandle) Get1Resource( 'styl', kStyleScrapID);
			if (theStyles)
				DetachResource( (Handle) *theStyles);
			
			itsResFork->Close();
			
			UseResFile( savedRes);
		}
		else
			isStyledText = FALSE;
	}
	CATCH
	{
		ForgetHandle( *theText);
		ForgetHandle( *theStyles);
	}
	ENDTRY;
}

/******************************************************************************
 InstallData
 
 	This method installs the text and styles into the window. It is called
 	by OpenFile and DoRevert. This method assumes that the caller will
 	dispose of the text and styles, CEditText and CStyleText make
 	copies of the data.
 	
 ******************************************************************************/


void CTextDoc::InstallData( Handle theText, StScrpHandle theStyles)
{
	LongPt			homePos;
	CAbstractText 	*itsTextPane;
	
	ASSERT( itsMainPane);
	
		// Install the text
		
	itsTextPane = (CAbstractText*) itsMainPane;
	if (theText)
		itsTextPane->SetTextHandle(theText);

		// Install the styles
		
	if (isStyledText && theStyles)
		((CStyleText*)itsMainPane)->SetStyleScrap( 0, MAXINT, theStyles, TRUE);
		
		// Scroll to the top
		
	itsTextPane->GetHomePosition( &homePos);
	itsTextPane->ScrollTo( &homePos, TRUE);
	itsTextPane->SetSelection( 0, 0, TRUE);
}

/******************************************************************************
 DoRevert {OVERRIDE}
 
 	Revert back to the previously saved version of the file. This method 
 	re-reads the data and calls InstallData to put it into the window.
 	
 ******************************************************************************/


void CTextDoc::DoRevert( void)
{
	Handle 			theText = NULL;
	StScrpHandle 	theStyles = NULL;
	
	TRY
	{
			// Commit the last task, because undo no longer makes sense
				
		Notify( NULL);
		
			// Read the data and styles and place it in the window
		
		ReadData( &theText, &theStyles);
		InstallData( theText, theStyles);
		
			// free the data and styles, InstallData makes copies.
		ForgetHandle( theText);
		ForgetHandle( theStyles);
		
		dirty = FALSE;
	}
	CATCH
	{
		ForgetHandle( theText);
		ForgetHandle( theStyles);	
	}
	ENDTRY;
}

/******************************************************************************
 Dispose {OVERRIDE}
 
 	You must override to dispose to free any memory you allocate in a subclass.
 ******************************************************************************/

void CTextDoc::Dispose( void)
{
	ForgetObject( itsResFork);
	inherited::Dispose();
}