/******************************************************************************
 CObject.c

							The root-level Object Class
	
		Abstract class which is at the root of the class hierarchy. CObject
		is the only class which does not have a superclass.	

	SUPERCLASS = None
	
	Copyright  1989 Symantec Corporation. All rights reserved.
	
	TCL 1.1 CHANGES
	[
		- implement Lock method.
		- Add failure handling
	]

 ******************************************************************************/

#include "Global.h"
#include "CObject.h"
#include "TCLUtilities.h"

class CApplication;
extern CApplication *gApplication;

/******************************************************************************
 Dispose

		Dispose of an object by freeing the block of memory it occupies
 ******************************************************************************/

void	CObject::Dispose()
{
	delete(this);						/* Call built-in delete function	*/
}


/******************************************************************************
 Copy

		Make a copy of an object
 ******************************************************************************/

CObject*	CObject::Copy()
{
	Handle		copyOfObject;
	OSErr		err;
	Boolean		savedAlloc;
	
	savedAlloc = SetAllocation( kAllocCanFail);
	
	copyOfObject = (Handle) this;		/* Objects are just handles			*/
	err = HandToHand(&copyOfObject);
	
	SetAllocation( savedAlloc);
	FailOSErr( err);
	
	return((CObject*) copyOfObject);
}

/******************************************************************************
 Lock
 
 	When fLocks is TRUE, makes an object non-relocatable, when FALSE
 	makes it relocatable, returns TRUE if previously locked.
******************************************************************************/

Boolean CObject::Lock( Boolean fLock)
{
	#define kLockMask	0x80	/* bit 7 of handle state is set when locked */
	SignedByte state;
	Boolean locked;
	
										/* test lock bit of handle state */
	locked = (HGetState((Handle) this) & kLockMask) != 0;
	
	if (locked != fLock)
	{
		if (fLock) 
			HLock( (Handle) this);
		else
			HUnlock( (Handle) this);
	}
	return locked;
	
}	/* CObject::Lock */
	
/******************************************************************************
 GetClassName
 
 ******************************************************************************/

void CObject::GetClassName( Str255 className)
{
	char *name = class_name( this);
	long len = cstrlen( name);
	
	len = Min( len, sizeof(Str255));
	BlockMove( name, &className[1], len);
	Length( className) = len;
}

/******************************************************************************
 SubclassResponsibility
 
 ******************************************************************************/

void CObject::SubclassResponsibility()
{
#ifdef __TCL_DEBUG__

	char *className = class_name( this);
	
	// +++ Get class name and method name, 
	// do alert "CFoo didn't override method Bar"
	
	DebugStr( (StringPtr) "\pSubclassResponsibility");

#endif
}

/******************************************************************************
 operator new
 
 ******************************************************************************/

void*	CObject::operator new( long objSize)
{
	Handle objHandle;
	Boolean	savedAlloc;
	
	if (gApplication)
	{
		savedAlloc = SetAllocation( kAllocCanFail);

		objHandle = NewHandleClear( objSize);
		
		SetAllocation( savedAlloc);
	}
	else
		objHandle = NewHandleClear( objSize);
	
	FailNIL( objHandle);
		
	return objHandle;
}

/******************************************************************************
 operator delete
 
 ******************************************************************************/

void CObject::operator delete( void* objStorage)
{
	ASSERT( objStorage != NULL);
	
	DisposHandle( objStorage);
}
