/******************************************************************************
 CPicture.c

							The Picture Class
		
		Displays a standard Toolbox "picture".
	
	SUPERCLASS = CPanorama
	
	Copyright  1989 Symantec Corporation. All rights reserved.

	TCL 1.1 CHANGES
	[
		- Added exception handling
		- Dispose only disposes of the picture if it is not a resource.
	]
 ******************************************************************************/

#include "Global.h"
#include "CPicture.h"
#include "TCLUtilities.h"
	
/******************************************************************************
 IPicture

		Initialize a Picture object
 ******************************************************************************/

void	CPicture::IPicture(
	CView			*anEnclosure,
	CBureaucrat		*aSupervisor,
	short			aWidth,
	short			aHeight,
	short			aHEncl,
	short			aVEncl,
	SizingOption	aHSizing,
	SizingOption	aVSizing)
{
	CPanorama::IPanorama(anEnclosure, aSupervisor,
					aWidth, aHeight, aHEncl, aVEncl, aHSizing, aVSizing);
	
	macPicture = NULL;
	scaled = FALSE;
	autoRefresh = scaled;
	isResPicture = FALSE;
	ownsPicture = FALSE;
}
	

/******************************************************************************
 IViewTemp {OVERRIDE}

		Initialize a Picture object using a template
 ******************************************************************************/

void	CPicture::IViewTemp(
	CView			*anEnclosure,		/* Enclosing view					*/
	CBureaucrat		*aSupervisor,		/* Boss in chain of command			*/
	Ptr				viewData)			/* Template with View data			*/
{
	register PictureTempP	p;
	
	p = (PictureTempP) viewData;		/* Initialize superclass			*/
	inherited::IViewTemp(anEnclosure, aSupervisor, (Ptr) &p->sPanoramaTemp);
	
	scaled = p->scaled != 0;			/* Set instance vars from template	*/
	autoRefresh = scaled;
	if (p->PICTid != NOTHING) {
		UsePICT(p->PICTid);
	} else {
		macPicture = NULL;
	}
}

/******************************************************************************
 Dispose  (OVERRIDE)

        Dispose of a Picture object
 ******************************************************************************/

void    CPicture::Dispose()
{
	if (macPicture && ownsPicture)
	{
		if (isResPicture)
			HPurge( (Handle) macPicture);
		else
    		KillPicture(macPicture);
    }
    macPicture = NULL;
    inherited::Dispose();
}
    

/******************************************************************************
 Draw {OVERRIDE}

		Draw a Picture
 ******************************************************************************/

void	CPicture::Draw(
	Rect		*area)
{
	Rect		tempRect;
	Boolean		savedAlloc;
	SignedByte	savedState;
	
	if (macPicture == NULL)
	{
		LongToQDRect( &aperture, &tempRect);
		EraseRect(&tempRect);
	
	}
	else 
	{
		if (isResPicture)
		{
			savedAlloc = SetAllocation( kAllocCanFail);
			LoadResource( (Handle) macPicture);
			SetAllocation( savedAlloc);
			FailResError();
			savedState = HGetState( (Handle) macPicture);
			HNoPurge( (Handle) macPicture);
		}
		if (scaled) 
		{
			LongToQDRect( &frame, &tempRect);
			DrawPicture(macPicture, &tempRect);
		} 
		else 
		{
			LongToQDRect( &bounds, &tempRect);
			DrawPicture(macPicture, &tempRect);
		}
		if (isResPicture)
			HSetState( (Handle) macPicture, savedState);
	}
}


/******************************************************************************
 SetMacPicture

	Specify a new PicHandle for a Picture. ownsPicture is set to true, unless
	the picture is from a purgeable resource. ownsPicture determines whether the
	picture is disposed or purged when the CPicture object is disposed.
 ******************************************************************************/

void	CPicture::SetMacPicture(
	PicHandle		aMacPicture)
{
	short 		resID;
	ResType		resType;
	Str255		resName;
	LongRect	newBounds;
	Boolean		purgeable;
	
	macPicture = aMacPicture;
	
	if (macPicture != NULL)
	{
		ownsPicture = TRUE;
		
		QDToLongRect( &(**macPicture).picFrame, &newBounds);
		OffsetLongRect(&newBounds, -newBounds.left, -newBounds.top);
		
			/* determine if this picture is from a resource		*/
			
		GetResInfo( (Handle) macPicture, &resID, &resType, resName);
		isResPicture = ResError() == noErr;
		if (isResPicture)
		{
			purgeable = GetResAttrs( (Handle) macPicture) & resPurgeable;
			if (purgeable)
				ownsPicture = FALSE;
		}
		else
			HNoPurge( (Handle) macPicture);
	} 
	else
	{
		SetLongRect(&newBounds, 0, 0, 0, 0);
		ownsPicture = FALSE;
	}
	SetBounds( &newBounds);
}


/******************************************************************************
 GetMacPicture

		Retrieve the PicHandle for a Picture
 ******************************************************************************/

PicHandle	CPicture::GetMacPicture()
{
	return(macPicture);					/* Return instance variable			*/
}


/******************************************************************************
 UsePICT

		Use a new picture specified by a PICT resource id
 ******************************************************************************/

void	CPicture::UsePICT(
	short		PICTid)
{
	PicHandle pict;
	Boolean	  savedAlloc;
	

	savedAlloc = SetAllocation( kAllocCanFail);
	pict = GetPicture( PICTid);
	SetAllocation( savedAlloc);

	FailNILRes( pict);
	SetMacPicture( pict);
}


/******************************************************************************
 SetScaled

		Set the scaled flag which controls whether the picture is scaled
		to the frame of the pane or drawn at its natural size
 ******************************************************************************/

void	CPicture::SetScaled(
	Boolean		aScaled)
{
	LongRect	pBounds;

	scaled = aScaled;
	
	if (scaled) {						/* Fit picture within frame			*/
		position.h = frame.left;		/* Top left of picture is at top	*/
		position.v = frame.top;			/*   left of the frame				*/
		SetBounds(&frame);				/* Boundaries are the same as frame	*/
		
	} else {							/* Display picture at normal size	*/
	
		QDToLongRect( &(**macPicture).picFrame, &pBounds);
		
										/* Put top left of bounds at (0, 0)	*/
		OffsetLongRect(&pBounds, -pBounds.left, -pBounds.top);
		SetBounds(&pBounds);
		SetPosition( &position);		/* Need to reposition picture since	*/
										/*   top left may have moved		*/
	}
	autoRefresh = scaled;				/* Size of frame affects picture	*/
										/*   only when it is scaled			*/
}


/******************************************************************************
 GetScaled

		Return whether or not a Picture is scaled
 ******************************************************************************/

Boolean	CPicture::GetScaled()
{
	return(scaled);						/* Return instance variable			*/
}

	
/******************************************************************************
 ResizeFrame {OVERRIDE}

		Adjust a Pictures's frame when the size of the pane changes. The delta
		rectangle parameter specifies the change in location of each side
		of the Pane. Positive values mean a move to the right or down;
		negative values mean a move to the left or up.

		After using the inherited method, we have to reset the bounds if
		the picture is scaled to the frame.
 ******************************************************************************/

void	CPicture::ResizeFrame(
	Rect		*delta)
{
	inherited::ResizeFrame(delta);
	
	if (scaled) {
		SetBounds(&frame);
	}
}


/******************************************************************************
 FrameToBounds

		Make the frame of the Picture the same size as the bounds
 ******************************************************************************/

void	CPicture::FrameToBounds()
{
	frame = bounds;
	width = frame.right - frame.left;
	height = frame.bottom - frame.top;
}

