/******************************************************************************
 CSwitchboard.c

							The Switchboard Class
		
		Retrieves events from the Toolbox event queue and passes messages
		to the proper objects for responding to them.
	
	SUPERCLASS = CObject
	
	Copyright  1989 Symantec Corporation. All rights reserved.
	
	TCL 1.1 CHANGES
	[
		- changed DoKeyEvent so non-menu commands keys are passed
		  along the command chain just like regular key downs
		- implemented DoHighLevelEvent and DoAppleEvent,GetAnEvent, DispatchEvent
		  methods.
		- implemented AppleEventHandler and AppleEventIdleProc
		  callbacks.
	]
	
 ******************************************************************************/

#include "Global.h"
#include "TBUtilities.h"
#include "CApplication.h"
#include "CDesktop.h"
#include "CBartender.h"
#include "CWindow.h"
#include "CSwitchboard.h"
#include "Commands.h" 
#include "Constants.h"
#include "CAppleEvent.h"
#include "AppleEvents.h"

#include "EPPC.h"

/*** Global Variables ***/
	
extern CApplication	*gApplication;		/* Application object				*/
extern CDesktop		*gDesktop;			/* The visible Desktop				*/
extern CBartender	*gBartender;		/* Manages all menus				*/
extern CBureaucrat	*gGopher;			/* First in line to get commands	*/
extern EventRecord	gLastMouseDown;		/* Previous mouse down event		*/
extern EventRecord	gLastMouseUp;		/* Previous mouse up event			*/
extern CView		*gLastViewHit;		/* Last view clicked in				*/
extern long			gSleepTime;			/* Max time between events			*/

/*** Class Constants ***/

#define		MouseMovedEvt		0xFA	/* Mouse moved event code			*/
#define		SuspendResumeEvt	0x01	/* Suspend/Resume event code		*/
#define		ResumeEvtMask		0x1		/* Supend or Resume selector		*/
#define		ConvertScrapMask	0x2		/* Scrap conversion flag			*/


/******************************************************************************
 ISwitchboard

		Initialize a Switchboard object.
 ******************************************************************************/

void	CSwitchboard::ISwitchboard()
{
	
	mouseRgn = NewRgn();				/* Allocate space for mouse region	*/
	
				// Install a single handler for all application AppleEvents
			
	if (gSystem.hasAppleEvents)
	{	
		FailOSErr( 
			AEInstallEventHandler( typeWildCard, typeWildCard, 
						(EventHandlerProcPtr) AppleEventHandler,
						0, FALSE)
				 ); 
	}
}	


/******************************************************************************
 DoMouseDown

		Mouse button was pressed. Save event and send message to the
		Desktop. The Desktop encompasses the entire screen, so click
		must be inside it.
 ******************************************************************************/

void	CSwitchboard::DoMouseDown(
	EventRecord		*macEvent)			/* The mouse down event				*/
{
	gDesktop->DispatchCursor( macEvent->where, mouseRgn);
	
	gDesktop->DispatchClick(macEvent);	/* Let Desktop handle it			*/
	gLastMouseDown = *macEvent;			/* Save for multi-click checking	*/
}


/******************************************************************************
 DoMouseUp

		Mouse button was released. Save event and send message to the
		view which was hit by the preceding mouse down. It doesn't matter
		where the mouse up occurred. The view hit by the associated mouse
		down (there must be a mouse down before a mouse up can happen) gets
		to handle it.
 ******************************************************************************/

void	CSwitchboard::DoMouseUp(
	EventRecord		*macEvent)			/* The mouse up event				*/
{
	if (gLastViewHit != NULL)			/* Let target view handle it		*/
		gLastViewHit->DoMouseUp(macEvent);
	gLastMouseUp = *macEvent;			/* Save for multi-click checking	*/
}


/******************************************************************************
 DoKeyEvent

		Process a keyboard event
 ******************************************************************************/

void	CSwitchboard::DoKeyEvent(
	register EventRecord	*macEvent)	/* The keyboard event				*/
{
	char			theChar;			/* The associated character			*/
	Byte			keyCode;			/* Code for the associated key		*/
	register long	menuChoice;			/* Menu equivalent key command		*/
	
    
    short           menuID, itemNo;
    MenuHandle      macMenu;
    
	theChar = macEvent->message & charCodeMask;
	keyCode = (macEvent->message & keyCodeMask) >> 8;
	
	switch (macEvent->what) {
	
		case keyDown:					/* Key was pressed					*/
										/* Check state of the command key	*/
			if (macEvent->modifiers & cmdKey) {
	
										/* Command key was down. Extract	*/
										/*   the character and check for a	*/
										/*   keyboard equivalent for a menu	*/
										/*   selection.						*/
				gBartender->UpdateAllMenus();
				menuChoice = MenuKey(theChar);
										/* Handle menu command or pass it	*/
										/* to the gopher if not a menu choice */
				if (HiShort(menuChoice)) {
					gGopher->DoCommand(gBartender->FindCmdNumber
									(HiShort(menuChoice), LoShort(menuChoice)));
					HiliteMenu(0);		/* Unhighlight the menu title		*/
				}
				else
				{		/* pass non-menu commands key up the command chain */
				
					gGopher->DoKeyDown(theChar, keyCode, macEvent);
				}
			} else
                        
            switch (keyCode) {
            
                case KeyF1:                         /* perform Undo command */
                    gBartender->UpdateAllMenus();
                    gBartender->FindMenuItem(cmdUndo, &menuID, &macMenu, &itemNo);
                    if (((*macMenu)->enableFlags & 1) &&
                        (((*macMenu)->enableFlags >> itemNo) & 1)) {
                        
                        HiliteMenu(menuID);
                        gGopher->DoCommand(cmdUndo);
                        HiliteMenu(0);
                    }
                    break;
                    
                case KeyF2:                         /* perform Cut command */
                    gBartender->UpdateAllMenus();
                    gBartender->FindMenuItem(cmdCut, &menuID, &macMenu, &itemNo);
                    if (((*macMenu)->enableFlags & 1) &&
                        (((*macMenu)->enableFlags >> itemNo) & 1)) {
                        
                        HiliteMenu(menuID);
                        gGopher->DoCommand(cmdCut);
                        HiliteMenu(0);
                    }
                    break;
                    
                case KeyF3:                         /* perform Copy command */
                    gBartender->UpdateAllMenus();
                    gBartender->FindMenuItem(cmdCopy, &menuID, &macMenu, &itemNo);
                    if (((*macMenu)->enableFlags & 1) &&
                        (((*macMenu)->enableFlags >> itemNo) & 1)) {
                        
                        HiliteMenu(menuID);
                        gGopher->DoCommand(cmdCopy);
                        HiliteMenu(0);
                    }
                    break;
                    
                case KeyF4:                         /* perform Paste command */
                    gBartender->UpdateAllMenus();
                    gBartender->FindMenuItem(cmdPaste, &menuID, &macMenu, &itemNo);
                    if (((*macMenu)->enableFlags & 1) &&
                        (((*macMenu)->enableFlags >> itemNo) & 1)) {
                        
                        HiliteMenu(menuID);
                        gGopher->DoCommand(cmdPaste);
                        HiliteMenu(0);
                    }
                    break;
                    
                default:
                        
                                        /* A regular keystroke; let the     */
                                        /*   gopher handle it.              */
                    gGopher->DoKeyDown(theChar, keyCode, macEvent);
                    break;
            }
			break;
			
		case keyUp:						/* Key was released					*/
			gGopher->DoKeyUp(theChar, keyCode, macEvent);
			break;
			
		case autoKey:					/* Key was held down				*/
			gGopher->DoAutoKey(theChar, keyCode, macEvent);
			break;
	}
}


/******************************************************************************
 DoDiskEvent

		A disk was inserted.
 ******************************************************************************/

void	CSwitchboard::DoDiskEvent(
	EventRecord		*macEvent)			/* The disk event					*/
{
	Point		corner;					/* Location for Dialog box			*/
	
	if (HiWord(macEvent->message) != noErr) {
										/* Some sort of error occurred.		*/
										/*   Check if user wants to format	*/
										/*   the disk.						*/
										
		SetPt(&corner, 112, 80);		/* ??? Can dialog box be centered	*/
										/*   There doesn't appear to be a	*/
										/*   DLOG resource for this one.	*/
										/*   Location (112,80) is the value	*/
										/*   used by Finder 6.1. 			*/
		DIBadMount(corner, macEvent->message);
	}
}


/******************************************************************************
 DoUpdate

		The contents of a window must be redrawn.
 ******************************************************************************/

void	CSwitchboard::DoUpdate(
	EventRecord		*macEvent)			/* The update event					*/
{
	(EventToWindow(macEvent))->Update();
}


/******************************************************************************
 DoActivate

		A window needs to be activated.
 ******************************************************************************/

void	CSwitchboard::DoActivate(
	EventRecord		*macEvent)			/* The activate event				*/
{
	(EventToWindow(macEvent))->Activate();
}


/******************************************************************************
 DoDeactivate

		A window needs to be deactivated.
 ******************************************************************************/

void	CSwitchboard::DoDeactivate(
	EventRecord		*macEvent)			/* The deactivate event				*/
{
	(EventToWindow(macEvent))->Deactivate();
}


/******************************************************************************
 DoSuspend

		This application is about to be switched into the background under
		MultiFinder.
 ******************************************************************************/

void	CSwitchboard::DoSuspend(
	EventRecord		*macEvent)			/* The suspend event				*/
{
	gApplication->Suspend();			/* Suspend the application			*/
}


/******************************************************************************
 DoResume

		This application is about to be switched into the foreground under
		MultiFinder.
 ******************************************************************************/

void	CSwitchboard::DoResume(
	EventRecord		*macEvent)			/* The resume event					*/
{
	gApplication->Resume();				/* Resume the application			*/
}


/******************************************************************************
 DoOtherEvent

		Handle some uncommom event. For the moment, this includes the
		networkEvt (which Apple says you should not use), and the
		user-defined events app1Evt, app2Evt, and app3Evt (Apple may
		claim these events in future systems, as they have done by
		using app4Evt for MultiFinder-related events.
 ******************************************************************************/

void	CSwitchboard::DoOtherEvent(
	EventRecord		*macEvent)			/* The other event					*/
{
}										/* NULL Method						*/


/******************************************************************************
 DoIdle

		No event directed to our application is pending. Here's our chance
		to perform periodic tasks. The classic example is flashing a
		text insertion cursor.
 ******************************************************************************/

void	CSwitchboard::DoIdle(
	EventRecord		*macEvent)			/* Usually a null or system event	*/
{
	gApplication->Idle(macEvent);
}


/******************************************************************************
 ProcessEvent

		Get the next event in the queue and call the proper method to
		handle it.
		
	TCL 1.1: Previously, ProcessEvent get the event itself and did its own
			 dispatching. It now calls GetAnEvent to get an event from
			 the queue and DispatchEvent to handle the event. This
			 makes the event dispatch mechanism easier to modify, as
			 well as making integration with AppleEvents easier.
 ******************************************************************************/

void	CSwitchboard::ProcessEvent( void)
{
	EventRecord			macEvent;		/* Mac Event to be processed		*/
	Boolean				isMyEvent;		/* Should program handle event?		*/
	Point				mouseLoc;		/* Mouse location in global coords	*/
	
		/* Make sure our cursor shape and mouse region are set properly	*/
		/* before calling WaitNextEvent. We don't want to be stuck with	*/
		/* the wrong cursor shape if a minor switch occurs (i.e., a		*/
		/* background process get some cpu time). We also set the		*/
		/* cursor shape before handling an event. This ensures that the	*/
		/* the shape is correct even if the user performs some rapid	*/
		/* mouse movement and clicking.									*/
        
	gDesktop->Prepare();
    GetMouse(&mouseLoc);

    gDesktop->DispatchCursor(mouseLoc, mouseRgn);

										/* Get an event from the queue		*/
	isMyEvent = GetAnEvent( &macEvent);
	
	if (isMyEvent) 						/* We must respond to this event	*/
	{			
		DispatchEvent( &macEvent);		
	} 
	else 
	{									/* No action required				*/
		DoIdle( &macEvent);
	}
}

/******************************************************************************
 GetAnEvent

		Get the next event in the event queue. If you want to peek at an event
		before the TCL sees it, override GetAnEvent, call inherited::GetAnEvent,
		and do what you want with the event before you return.
		
 ******************************************************************************/

Boolean	CSwitchboard::GetAnEvent( EventRecord *macEvent)
{
	Boolean eventResult;
	
	if (gSystem.hasWNE) 
	{
						/* WaitNextEvent is the preferred	*/
						/*   call.  It handles multitasking	*/
						/*   in a friendly manner.			*/
										
		eventResult = WaitNextEvent( everyEvent, macEvent,
									gSleepTime, mouseRgn);
		
	} 
	else 
	{
						/* WaitNextEvent is not available.	*/
						/*   We must use the old method of	*/
						/*   calling SystemTask and			*/
						/*   GetNextEvent.					*/
		SystemTask();
		eventResult = GetNextEvent( everyEvent, macEvent);
	}
	return eventResult;
}

/******************************************************************************
 DispatchEvent

	Dispatch the event to the appropriate handler.
 ******************************************************************************/

void	CSwitchboard::DispatchEvent( EventRecord *macEvent)
{
								

	switch (macEvent->what) {		/* Branch of the type of event		*/
	
							/** USER ACTIONS **/
		
		case mouseDown:				/* Mouse was pressed				*/
			DoMouseDown(macEvent);
			break;
			
		case mouseUp:				/* Mouse was released				*/
			DoMouseUp( macEvent);
			break;
			
		case keyDown:				/* Key was pressed					*/
		case keyUp:					/* Key was released					*/
		case autoKey:				/* Key was held down				*/
			DoKeyEvent( macEvent);
			break;
			
		case diskEvt:				/* Disk insertion Event				*/
			DoDiskEvent( macEvent);
			break;
			
							/** WINDOW MANAGER EVENTS **/
			
		case updateEvt:				/* Update window contents			*/
			DoUpdate( macEvent);
			break;
			
		case activateEvt:			/* Activate or Deactivate a window	*/
									/*   Check the modifiers field to	*/
									/*   determine which one it is		*/
			if (macEvent->modifiers & activeFlag)
				DoActivate( macEvent);
			else
				DoDeactivate( macEvent);
			break;
			
							/** MULTIFINDER EVENTS **/
		case osEvt:
		
									/* Event type is stored in the		*/
									/*   high byte of the event message	*/
									/*   Typecast to unsigned long so	*/
									/*   we won't get sign extension	*/
									/*   when the bits are shifted		*/
			switch ((unsigned long)macEvent->message >> 24) {
			
				case MouseMovedEvt:
									/* Since we adjust the cursor shape	*/
									/*   before handling an event, we	*/
									/*   can just idle here				*/
					DoIdle( macEvent);
					break;
					
				case SuspendResumeEvt:
									/* Context switch					*/
					if (macEvent->message & ResumeEvtMask)
						DoResume( macEvent);
					else
						DoSuspend( macEvent);
					break;
			}
			break;
			
		case kHighLevelEvent:
			DoHighLevelEvent( macEvent);
			break;
			
		default:					/* Some other event					*/
			DoOtherEvent( macEvent);
			break;
	}
}

/******************************************************************************
 DoHighLevelEvent
 
 	Handle a high level events. The TCL assumes all high level
 	events are AppleEvents. If you use high level events that
 	don't follow the Apple Event Interprocess Messaging Protocol you
 	must override DoHighLevelEvent and handle those events yourself.
 	Be sure to call inherited::DoHighLevelEvent so standard AppleEvents
 	get handled.
 	
******************************************************************************/

void CSwitchboard::DoHighLevelEvent( const EventRecord *theEvent)
{
	OSErr	err;
	if (gSystem.hasAppleEvents)
	{
		err = AEProcessAppleEvent( theEvent);
	
		// +++ what to do with error? We return error codes in the
		// reply, but should the server app report an error?
		// client may either be this app, another app on this machine,
		// or an app on another machine.	
	}
	
}

/******************************************************************************
 DoAppleEvent
 
  	Respond to an AppleEvent. The AppleEvent and the default reply are
  	packaged into a CAppleEvent object and sent to the gopher. If an exception
  	occurs, the error causing the exception is return as the result to the
  	operating system. The exception does not propagate beyond this method.
  	
  	This method calls CApplication::PackageAppleEvent to package the event
  	and its default reply into a CAppleEvent object. Override that method
  	if you need to create a custom subclass of CAppleEvent.
  	
******************************************************************************/

OSErr CSwitchboard::DoAppleEvent( struct AppleEvent *theEvent, 
					struct AppleEvent *theReply, long refCon)
{
	OSErr		err;
	CAppleEvent	*appleEvent = NULL;
	DescType	actualType;
	Size		actualSize;
	DescType	eventClass, eventID;
		
	TRY
	{
			// Get the event class
			                      
		FailOSErr( AEGetAttributePtr( (AppleEvent*) theEvent, keyEventClassAttr,
					typeType, &actualType, (Ptr) &eventClass, 
					sizeof(eventClass), &actualSize));
							
			// Get the event ID
	
		FailOSErr( AEGetAttributePtr(  (AppleEvent*) theEvent, keyEventIDAttr,
					typeType, &actualType, (Ptr) &eventID, 
					sizeof(eventID), &actualSize));
									
		appleEvent = gApplication->PackageAppleEvent( theEvent, 
							theReply, refCon, eventClass, eventID);
	
		gGopher->DoAppleEvent( appleEvent);
		
		// When the CAppleEvent is initialized, it sets its errorCode to 
		// errAEEventNotHandled. If no one handles the event, the error code
		// will not be modified, and we will correctly signal that the
		// event wasn't handled. If it was handled successfully, then
		// whoever handled it should have marked it as such by calling
		// appleEvent->SetErrorResult( noErr). If an error occurred in handling
		// it, then either someone should have called 
		// appleEvent->SetErrorResult( theError), or an exception occurred and
		// we pick up the error in the CATCH block below.
		
		err = appleEvent->GetErrorResult();
				
		ForgetObject( appleEvent);
	}
	CATCH
	{
		err = gLastError;
		ForgetObject( appleEvent);
		
		NO_PROPAGATE;
	}
	ENDTRY;
	
	return err;
}

/******************************************************************************
 AppleEventIdle
 
	Implements the IdleProc used by AEInteractWithUser and AESend. It handles
	events received while we're waiting for a reply or waiting for our process
	to be brought to the front. 
	
	According to the AppleEvent chapter of IM VI, theEvent may only
	be a null event, update event, OS event, or activate event. If it
	is a null event, we should return the desired sleep time and mouse region.
	
	We should return FALSE if we are willing to continue waiting, TRUE if
	we want to abort. This method checks for a cmd-'.' in the event queue
	and returns TRUE if it finds one.
	
******************************************************************************/

Boolean CSwitchboard::AppleEventIdle( EventRecord *theEvent, long *sleepTime, 
				RgnHandle *mouseRgn)
{
	Point	mouseLoc;
	
	if (theEvent->what == nullEvent)
	{
			// make sure mouseRgn is up-to-date
			
		gDesktop->Prepare();
		gDesktop->DispatchCursor( theEvent->where, this->mouseRgn);
		 
		 	// When DoIdle returns, gSleepTime is up-to-date 
		 	
		DoIdle( theEvent);	
	}
	else
	{
		DispatchEvent( theEvent);
	}
	
	*mouseRgn = this->mouseRgn;
	*sleepTime = gSleepTime;

	return AbortInQueue();
}


pascal OSErr AppleEventHandler( struct AppleEvent *event, struct AppleEvent *reply, long refCon)
{	
	return gApplication->itsSwitchboard->DoAppleEvent( event, reply, refCon);
}

pascal short AppleEventIdleProc( EventRecord *theEvent, long *sleepTime, 
				RgnHandle *mouseRgn)
{
	Boolean result = gApplication->itsSwitchboard->AppleEventIdle(
										theEvent, sleepTime, mouseRgn);
	
	return result? -1 : 0; // be cautious about returning a pascal Boolean
}