/******************************************************************************
 CWindow.c

							The Window Class
		
		A standard Mac window. Normally, all drawing within a window is
		performed by installing Panes in the window. This class implements
		the standard Mac behavior of overlapping windows which may be
		dragged, resized, zoomed, closed, etc.
	
	SUPERCLASS = CView
	
	Copyright  1989 Symantec Corporation. All rights reserved.
	
	TCL 1.1 CHANGES
	[
		- implemented IsModal, IsColor methods.
		- implemented INewWindow alternate initialization method
		- automatically creates a color window if Color QuickDraw
		  is available. The isColor instance variable is set to TRUE
		  if a color window was created. CColorWindow is still available
		  as a compatibility class.
	]

 ******************************************************************************/

#include "Global.h"
#include "Constants.h"
#include "CDesktop.h"
#include "CDirector.h"
#include "CPane.h"
#include "CWindow.h"
#include "TBUtilities.h"
#include "OSChecks.h"
#include "CApplication.h"
#include "CList.h"
#include "TCLUtilities.h"


typedef struct	WINDtemplate {			/* Format of a WIND resource		*/
	Rect		boundsRect;
	short		WDEFid;
	short		visible;
	short		goAwayFlag;
	long		refCon;
	Str255		title;
} WINDtemplate, *WINDtemplateP, **WINDtemplateH;

extern CApplication	*gApplication;

/**** C O N S T R U C T I O N / D E S T R U C T I O N   M E T H O D S ****/


/******************************************************************************
 IWindow

		Initialize a Window object
 ******************************************************************************/

void	CWindow::IWindow(
	short			WINDid,				/* Window resource ID				*/
	Boolean			aFloating,			/* Is this a floating window?		*/
	CDesktop		*anEnclosure,		/* Enclosing view of the window		*/
	CDirector		*aSupervisor)		/* Supervisor in chain of command	*/
{
	register WINDtemplateH	theWIND;	/* Handle to WIND resource template	*/
	short					saveVis;	/* Oringal setting of visible flag	*/

										/* Initialize superclass.			*/
	inherited::IView(anEnclosure, aSupervisor);
	
		/* Because we maintain our own window list and have to do	*/
		/* a lot of manual activating/deactivating to support		*/
		/* floating windows, it is imperative that windows be		*/
		/* invisible when initially created. Therefore, we read the	*/
		/* the WIND resource template and set the visible flag to	*/
		/* FALSE (after saving the original value).					*/

	theWIND = (WINDtemplateH) GetResource('WIND', WINDid);
	FailNILRes( theWIND);
	saveVis = (**theWIND).visible;
	(**theWIND).visible = FALSE;
	
	procID = (**theWIND).WDEFid;
	
	floating = aFloating;				/* Set floating window flag			*/

	MakeMacWindow(WINDid);				/* Create Toolbox window record		*/

	IWindowX();	
		
	if (saveVis) {						/* If the WIND template had the		*/
										/*   visible flag set to TRUE ...	*/
										/* Get handle to the WIND template	*/
										/*   Call GetResource again since	*/
										/*   it could have been purged		*/
		theWIND = (WINDtemplateH) GetResource('WIND', WINDid);
		(**theWIND).visible = TRUE;		/* Reset flag to TRUE				*/
		Select();						/* Select will show the window and	*/
										/*   make it the active one			*/
	}
}

/******************************************************************************
 INewWindow
 
 	Alternate initialization method to dynamically specify the window 
 	parameters, rather than obtaining them from a resource.
******************************************************************************/

void CWindow::INewWindow( Rect *bounds, Boolean fVisible, short aProcID, 
 					Boolean fFloating, Boolean fHasGoAway,
					CDesktop *anEnclosure, CDirector *aSupervisor)
{
	CView::IView( anEnclosure, aSupervisor);
	
	floating = fFloating;
	procID = aProcID;
	
	MakeNewMacWindow( bounds, aProcID, fHasGoAway);
	
	IWindowX();
	
	if (fVisible) Select();
}

/******************************************************************************
 IWindowX
 
 	Perform initialization commond to IWindow and INewWindow
******************************************************************************/

void CWindow::IWindowX( void)
{
	isModal = FALSE;
	helpResID = kDefaultHelpResID;	// default resource ID for help balloon
									// 'hrct' resource
	
	SetWRefCon(macPort, (long) this);	/* Stuff handle to window object	*/
										/*   in reference constant field	*/
	((WindowPeek)macPort)->windowKind = OBJ_WINDOW_KIND;
	
										/* Let window grow as big as the	*/
										/*   screen and as small as some	*/
										/*   minimum size (a constant)		*/
	sizeRect = (**GetGrayRgn()).rgnBBox;
	
		/* Offset the sizeRect to normalize the rect to 0,0	*/
		/* Thanks to Bernie Bernstein for the fix			*/
		
	OffsetRect( &sizeRect, -sizeRect.left, -sizeRect.top);
	sizeRect.left = sizeRect.top = MIN_WSIZE;
	
	SetPt( &location, MAXINT, MAXINT);	/* set to "invalid" location		*/
	
										/* DEFAULT SETTINGS:				*/
	actClick = FALSE;					/* Does not process activate click	*/
	wantsClicks = TRUE;					/* Does handle mouse clicks			*/

										/* Stick window in our desktop		*/
	((CDesktop*)itsEnclosure)->AddWind(this);


}

/******************************************************************************
 Dispose {OVERRIDE}

		Get rid of a window and all its panes
 ******************************************************************************/

void	CWindow::Dispose()
{
	if (visible) {						/* Hide window first so the user	*/
		this->Hide();					/*   doesn't see the ugly details	*/
	}									/*   as each subview is disposed	*/
		
	if (itsSubviews != NULL) {			/* If this window has subviews ...	*/
		itsSubviews->DisposeAll();		/*   Throw them out					*/
		itsSubviews = NULL;				/* Clear dangling handle			*/
	}

										/* Remove window from desktop		*/
	((CDesktop*)itsEnclosure)->RemoveWind(this);
	itsEnclosure = NULL;

	DisposeWindow(macPort);				/* Kill the Toolbox window stuff	*/
	macPort = NULL;
	
	inherited::Dispose();				/* Pass message on to superclass	*/
}
	

/******************************************************************************
 MakeMacWindow

		Create a new Toolbox window record using the specified WIND
		resource ID. This method creates a normal Mac window and lets
		the window manager allocate space for the window record. Subclasses
		should override this method to perform their own memory allocation
		or to create a color window. For compatibility with the Desktop
		class, the convention of putting floating windows in front and
		non-floating windows in back should be observed.
 ******************************************************************************/

void	CWindow::MakeMacWindow(
	short		WINDid)					/* Window resource ID				*/
{
	Boolean savedAlloc;
										/* Create a new Mac window			*/
										/*   Floating windows are put in	*/
										/*   front of all others, regulars	*/
										/*   are put behind all others		*/
						
	savedAlloc = SetAllocation( kAllocCanFail);				
	
	isColor = gSystem.hasColorQD;
	
	if (isColor)
		macPort = GetNewCWindow(WINDid, NULL, (floating ? (WindowPtr)(-1L) : NULL));
	else
		macPort = GetNewWindow(WINDid, NULL, (floating ? (WindowPtr)(-1L) : NULL));
		
	SetAllocation( savedAlloc);
	
	FailNIL( macPort);
}
	
/******************************************************************************
 MakeNewMacWindow
 
 	Alternate window creation method whic creates window using dynamic
 	parameters rather than from a 'WIND' resource. This method creates a 
 	normal Mac window and lets the window manager allocate space for the 
 	window record. Subclasses should override this method to perform their
 	own memory allocation or to create a color window. For compatibility
 	with the Desktop class, the convention of putting floating windows in
 	front and non-floating windows in back should be observed.
******************************************************************************/

void CWindow::MakeNewMacWindow( Rect *bounds, short aProcID, Boolean fHasGoAway)
{
	Str31	wTitle;
	Boolean savedAlloc;
	
	Length(wTitle) = 0;
	
	savedAlloc = SetAllocation( kAllocCanFail);				
	
	isColor = gSystem.hasColorQD;
	
	if (isColor)
		macPort = NewCWindow( NULL, bounds, wTitle, FALSE, aProcID, 
				(floating ? (WindowPtr)(-1L) : NULL), fHasGoAway, 0);
	else
		macPort = NewWindow( NULL, bounds, wTitle, FALSE, aProcID, 
				(floating ? (WindowPtr)(-1L) : NULL), fHasGoAway, 0);

	SetAllocation( savedAlloc);
	
	FailNIL( macPort);

}

/******************************************************************************
 Close

		Respond to a click in the close box
 ******************************************************************************/

void	CWindow::Close()
{
	((CDirector*)itsSupervisor)->CloseWind(this);
}


/**** A C C E S S I N G   M E T H O D S ****/

	
/******************************************************************************
 GetFrame {OVERRIDE}

		Return the frame of the window
 ******************************************************************************/

void	CWindow::GetFrame(
	register LongRect		*theFrame)
{
										/* Frame of Window includes a one	*/
										/*   pixel border around its port	*/
										/*   rectangle						*/
	theFrame->left = theFrame->top = -1;
	theFrame->right = macPort->portRect.right - macPort->portRect.left + 1;
	theFrame->bottom = macPort->portRect.bottom - macPort->portRect.top + 1;
}

	
/******************************************************************************
 GetInterior {OVERRIDE}

		Return the interior of the window
 ******************************************************************************/

void	CWindow::GetInterior(
	register LongRect		*theInterior)
{
										/* Window interior is the same		*/
										/*   size as its port rectangle		*/
	theInterior->left = theInterior->top = 0;
	theInterior->right = macPort->portRect.right - macPort->portRect.left;
	theInterior->bottom = macPort->portRect.bottom - macPort->portRect.top;
}

	
/******************************************************************************
 GetAperture {OVERRIDE}

		Return the area of the window where drawing may occur. For a
		window, this is the entire port rectangle. The top left of the
		window is always at (0, 0), so the right side is the width and
		the bottom side is the height of the port rectangle.
 ******************************************************************************/

void	CWindow::GetAperture(
	register LongRect		*theAperture)
{
	theAperture->left = theAperture->top = 0;
	theAperture->right = macPort->portRect.right - macPort->portRect.left;
	theAperture->bottom = macPort->portRect.bottom - macPort->portRect.top;
}
	

/******************************************************************************
 IsFloating
 
		Check if a window is a floating window
 ******************************************************************************/

Boolean		CWindow::IsFloating()
{
	return(floating);					/* Return instance variable			*/
}

/******************************************************************************
 IsModal
 
		Check if a window is isModal
 ******************************************************************************/

Boolean		CWindow::IsModal()
{
	return isModal;			/* Return instance variable			*/
}

/******************************************************************************
 IsColor
 
 		Check if a window is a color window
******************************************************************************/

Boolean CWindow::IsColor( void)
{
	return isColor;					/* Return instance variable			*/

}	
/******************************************************************************
 SetModal
 	
 		Specify that a window become isModal if fModal is TRUE or modeless
 		if fModal is FALSE.
******************************************************************************/

void		CWindow::SetModal( Boolean fModal)
{
	isModal = fModal;
}

/******************************************************************************
 SetTitle

		Specify a title for a window
 ******************************************************************************/

void	CWindow::SetTitle(
	Str255		theTitle)
{
	SetWTitle(macPort, theTitle);		/* Toolbox call does the work		*/
}
	

/******************************************************************************
 GetTitle

		Return the title of a window
 ******************************************************************************/

void	CWindow::GetTitle(
	Str255		theTitle)
{
	GetWTitle(macPort, theTitle);		/* Toolbox call does the work		*/
}


/******************************************************************************
 SetActClick
 
		Specify the whether the window processes the mouse click which
		activates the window
 ******************************************************************************/

void	CWindow::SetActClick(
	Boolean		anActClick)
{
	actClick = anActClick;				/* Set instance variable			*/
}
	

/******************************************************************************
 WantsActClick
 
		Check if a window processes a click which activates the window
 ******************************************************************************/

Boolean		CWindow::WantsActClick()
{
	return(actClick);					/* Return instance variable			*/
}

	
/******************************************************************************
 Contains {OVERRIDE}

		Check whether a Window contains a specified point. Point is
		specified in global coordinates. Window contains a point if it
		is inside the window's content region.
 ******************************************************************************/

Boolean		CWindow::Contains(
	Point			thePoint)
{
	return(PtInRgn(thePoint, ((WindowPeek)macPort)->contRgn));
}
	

/******************************************************************************
 SetSizeRect
 
		Set the rectangle which specifies the maximum and minimum dimensions
		of a Window. The left and top of this rectangle are the minimum
		width and height; the right and bottom are the maximums.
		
		This method also adjusts the standard zoom out state if necessary,
		so that zooming does not grow the window larger than the maximum size.
 ******************************************************************************/

void	CWindow::SetSizeRect(
	register Rect		*aSizeRect)
{
	WStateData	**zoomData;
	Rect		theStdState;

	sizeRect = *aSizeRect;				/* Set instance variable			*/
	
										/* Check if window supports zooming	*/
	if (((WindowPeek)macPort)->spareFlag) {
										/* Get standard state rectangle		*/
		zoomData = (WStateData**) ((WindowPeek)macPort)->dataHandle;
		theStdState = (**zoomData).stdState;
		
										/* Adjust width						*/
		if (aSizeRect->right < (theStdState.right - theStdState.left)) {
			theStdState.right = theStdState.left + aSizeRect->right;
		}
										/* Adjust height					*/
		if (aSizeRect->bottom < (theStdState.bottom - theStdState.top)) {
			theStdState.bottom = theStdState.top + aSizeRect->bottom;
		}
		
		(**zoomData).stdState = theStdState;
	}
}
	

/******************************************************************************
 SetStdState
 
		Set the size and location for the window when it is zoomed out.
		Calling routine should make sure that the window supports zooming
		by checking the spareFlag field of its Mac window record.
 ******************************************************************************/

void	CWindow::SetStdState(
	Rect		*aStdState)
{
	WStateData	**zoomData;
	
	zoomData = (WStateData**) ((WindowPeek)macPort)->dataHandle;
	(**zoomData).stdState = *aStdState;
}

/******************************************************************************
 SetHelpResID

	Set the helpResID instance variable. helpResID gives the resource ID
	of a 'hrct' resource used to display help balloons for all the
	panes in this window.
 ******************************************************************************/

void CWindow::SetHelpResID( short aResID)
{
	helpResID = aResID;
}

/******************************************************************************
 GetHelpResID

	return the resource ID of the 'hrct' resource used for panes in this window.
 ******************************************************************************/

short CWindow::GetHelpResID( void)
{
	return helpResID;
}

/**** A P P E A R A N C E    M E T H O D S ****/
	

/******************************************************************************
 Show {OVERRIDE}

		Make a window visible
 ******************************************************************************/

void	CWindow::Show()
{
	if (!visible) {						/* Do nothing if already visible	*/
		inherited::Show();				/* Superclass sets status			*/
										/* Tell desktop to make us visible	*/
		((CDesktop*)itsEnclosure)->ShowWind(this);
	}
}
	

/******************************************************************************
 Hide {OVERRIDE}

		Make a window invisible
 ******************************************************************************/

void	CWindow::Hide()
{
	if (visible) {						/* Do nothing if already invisible	*/
		inherited::Hide();				/* Superclass sets status			*/
										/* Tell desktop to hide us			*/
		((CDesktop*)itsEnclosure)->HideWind(this);
	}
}
	

/******************************************************************************
 Activate {OVERRIDE}

		Make a window active
 ******************************************************************************/

void	CWindow::Activate()
{
	if (!active) {						/* Do nothing if already active		*/
		HiliteWindow(macPort, TRUE);	/* Unhilite window's title bar		*/
		inherited::Activate();			/* Let superclass set status		*/
										/* Notify supervisor				*/
		((CDirector*)itsSupervisor)->ActivateWind(this);
	}
}
	

/******************************************************************************
 Deactivate {OVERRIDE}

		Make a window inactive
 ******************************************************************************/

void	CWindow::Deactivate()
{	
	if (active) {						/* Do nothing if already inactive	*/
		HiliteWindow(macPort, FALSE);	/* Unhilite title bar of window		*/
										/* Notify supervisor				*/
		((CDirector*)itsSupervisor)->DeactivateWind(this);
		inherited::Deactivate();		/* Let superclass set status		*/
	}
}
	

/******************************************************************************
 Select

		Select a window by bringing it to the front and making it active
 ******************************************************************************/

void	CWindow::Select()
{
										/* Tell enclosure to select us		*/
	((CDesktop*)itsEnclosure)->SelectWind(this);
	visible = TRUE;
}
	

/******************************************************************************
 ShowResume

		Make a window visible when the application resumes
 ******************************************************************************/

void	CWindow::ShowResume()
{
	Boolean	showOnResume = (location.h != MAXINT) && (location.v != MAXINT);
	
	if (!visible && showOnResume)
	{									/* Do nothing if already visible	*/
		inherited::Show();				/* Let superclass set status flags	*/
										/* Move window back to its original	*/
										/*   location. The HideSuspend		*/
										/*   method saved this location		*/
										/*   before moving the window		*/
										/*   offscreen.						*/
		MoveWindow(macPort, location.h, location.v, FALSE);
		
		SetPt( &location, MAXINT, MAXINT);
	}
}
	

/******************************************************************************
 HideSuspend

		Make a window invisible when the application suspends. This method
		"hides" the window by moving it offscreen. The original
		location is saved so we can put the window back when the
		application resumes.
 ******************************************************************************/

void	CWindow::HideSuspend()
{
	if (visible) {						/* Do nothing if already invisible	*/
		inherited::Hide();				/* Let superclass set status flags	*/
										
										/* Save location of top left corner	*/
										/*   of window in global coords		*/
		SetPort(macPort);
		ForceNextPrepare();
		location = topLeft(macPort->portRect);
		LocalToGlobal(&location);
		
		MoveOffScreen();
	}
}
	

/******************************************************************************
 ShowOrHide {OVERRIDE}

		Make a window visible or invisible, without performing (or
		generating) any activates or deactivates.
			showFlag = TRUE, show window
			showFlag = FALSE, hide window 
 ******************************************************************************/

void	CWindow::ShowOrHide(
	Boolean		showFlag)
{
	ShowHide(macPort, showFlag);
	visible = showFlag;
}


/**** S I Z E   A N D   L O C A T I O N   M E T H O D S ****/
	

/******************************************************************************
 Drag

		Drag a window on the desktop
 ******************************************************************************/

void	CWindow::Drag(
	EventRecord		*macEvent)			/* Mouse down event for drag		*/
{
										/* Desktop does the dragging		*/
	((CDesktop*)itsEnclosure)->DragWind(this, macEvent);
}
	

/******************************************************************************
 Resize

		Dynamically resize a window
 ******************************************************************************/
		
void	CWindow::Resize(
	EventRecord		*macEvent)			/* Mouse down event for resize		*/
{
	register long	newSize;			/* New width and height for window	*/
	register long	oldSize;			/* Current width and height			*/
	Rect			theRect;			/* Utility rectangle				*/
	
	theRect = macPort->portRect;		/* Record the current width and		*/
										/*   height of the window in a long	*/
										/*   with the height in the hi word	*/
										/*   and the width in the lo word	*/
	oldSize = (((long)theRect.bottom - theRect.top) << 16) +
				(theRect.right - theRect.left);
				
	theRect = sizeRect;					/* Store instance variable in local	*/
										/*   var in case memory moves		*/
										
	theRect.right += 1;					/* GrowWindow trap is off by one	*/
	theRect.bottom += 1;				/*   when growing window to its max	*/
										/*   size. Mostly likely caused by	*/
										/*   using the PinRect trap.		*/
			  	  
			  	  						/* Let Toolbox draw the grow image	*/
			  	  						/*   newSize has the new height in	*/
			  	  						/*   the hi word, width in lo word	*/
	newSize = GrowWindow(macPort, macEvent->where, &theRect);
	
		/* Check whether the size changed. Inside Mac says that a newSize	*/
		/* of zero means that the size is the same. However, it does not	*/
		/* say that newSize will be zero in all cases where the size		*/
		/* remains the same. Let's perform our own check of whether the		*/
		/* size has changed. 												*/

	if (newSize && (newSize != oldSize)) {
		ChangeSize(LoShort(newSize), HiShort(newSize));
	}
}


/******************************************************************************
 Zoom

		Zoom a window out to a standard size or in to its original size
 ******************************************************************************/

void	CWindow::Zoom(
	short		direction)				/* Zoom can be in or out			*/
{
	Rect		delta;					/* Change in size of window			*/
	
	Prepare();
	delta = macPort->portRect;			/* Save current window size			*/
	
	ClipRect(&delta);					/* White out contents of window		*/
	EraseRect(&delta);
										/* Toolbox call performs the zoom	*/
	ZoomWindow(macPort, direction, FALSE);
	
	if (itsSubviews) {					/* Adjust size of all subviews		*/
										/* Find change in window size		*/
		delta.right = macPort->portRect.right - delta.right;
		delta.bottom = macPort->portRect.bottom - delta.bottom;
		itsSubviews->DoForEach1((EachFunc1) Pane_AdjustToEnclosure, (long)&delta);
		InvalRect(&macPort->portRect);	/* Force redraw of entire window	*/
	}
	
	if (floating) {
		((CDesktop*)itsEnclosure)->SelectWind(this);
	}
}
	

/******************************************************************************
 Move

		Move a window to a new location by specifying the global coordinates
		at which to place the top left corner of the port rectangle of the
		window. Note that the window's title bar, if present, will lie above
		the specified vertical coordinate.
 ******************************************************************************/

void	CWindow::Move(
	short		hGlobal,				/* Horizontal position				*/
	short		vGlobal)				/* Vertical position				*/
{
										/* Toolbox call does all the work.	*/
										/*   The final FALSE parameter		*/
										/*   means that we don't want to	*/
										/*   bring the window to the front	*/
										/*   if it is an underlying window.	*/
	MoveWindow(macPort, hGlobal, vGlobal, FALSE);
}
	

/******************************************************************************
 ChangeSize

		Change the size of a window by specifying the width and height
		in pixels. Window cannot be made larger than the maximum size
		specified by its sizeRect instance variable.
 ******************************************************************************/

void	CWindow::ChangeSize(
	short		width,					/* New width for window in pixels	*/
	short		height)					/* New height for window in pixels	*/
{
	Rect		theRect;
	
	theRect.left = theRect.top = 0;		/* Save current window size			*/
	theRect.right = macPort->portRect.right - macPort->portRect.left;
	theRect.bottom = macPort->portRect.bottom - macPort->portRect.top;
	
										/* Constrain size to the max & min	*/
										/*   allowed for the window			*/
	width = Min(width, sizeRect.right);
	height = Min(height, sizeRect.bottom);
	width = Max(width, sizeRect.left);
	height = Max(height, sizeRect.top);

										/* Toolbox call does all the work.	*/
										/*   The final TRUE parameter means	*/
										/*   that we want any newly exposed	*/
										/*   area to be added to the update	*/
										/*   region of the window.			*/
	
	SizeWindow(macPort, width, height, TRUE);
	
	if (itsSubviews) {					/* Adjust size panes within window	*/
		theRect.right = width - theRect.right;
		theRect.bottom = height - theRect.bottom;
		itsSubviews->DoForEach1((EachFunc1) Pane_AdjustToEnclosure, (long)&theRect);
	}
    
    ForceNextPrepare();
    
    Update();   /* Update the window */
}
	

/******************************************************************************
 MoveOffScreen

		Move a window offscreen by placing the left edge of the window past
		the right side of the desktop.
 ******************************************************************************/

void	CWindow::MoveOffScreen()
{
	Rect	deskBounds;
	
		/* ??? Stick the top left of the port rectangle 1000 pixels to	*/
		/* the right and below the bottom right corner of the desktop.	*/
		/* This could cause a problem for windows with a structure		*/
		/* region larger than 1000 pixels than the content region or	*/
		/* for extremely large windows (on the order of 32000 pixels)	*/
		/* where moving the window may cause arithmetic overflow. Such	*/
		/* situations are very rare -- you really have to go out of		*/
		/* your way to come up with a practical reason for having		*/
		/* windows which are so large.									*/

	((CDesktop*)itsEnclosure)->GetBounds(&deskBounds);
	MoveWindow(macPort, deskBounds.right + 1000, deskBounds.bottom + 1000, FALSE);
}	


/**** D R A W I N G    M E T H O D S ****/


/******************************************************************************
 Update

		Update the contents of a window
 ******************************************************************************/

void	CWindow::Update()
{
	GrafPtr		savePort;				/* The current port					*/
	Rect		updateRect;				/* Bounding box of update region	*/
	
	GetPort(&savePort);					/* Save the original port			*/
	Prepare();							/* Set up Window's port				*/
	BeginUpdate(macPort);				/* Start the update process			*/
										/* This restricts the visible area	*/
										/*   to just the update region,		*/
										/*   meaning that no drawing will	*/
										/*   occur outside this region		*/

	
	ClipRect(&thePort->portRect);		/* Clip to the entire window		*/
	EraseRect(&thePort->portRect);		/* Erase the update region			*/
	
	if (itsSubviews != NULL) {			/* Draw all subviews				*/
		updateRect = (**thePort->visRgn).rgnBBox;
		itsSubviews->DoForEach1((EachFunc1) Pane_Draw, (long) &updateRect);
	}
	
	EndUpdate(macPort);					/* End the update process			*/
	SetPort(savePort);					/* Restore the original port		*/
	ForceNextPrepare();
}

	
/******************************************************************************
 Prepare {OVERRIDE}

		Set up the Window's port and coordinate system
 ******************************************************************************/

void	CWindow::Prepare()
{	
	SetPort(macPort);
	SetOrigin(0, 0);
	
	inherited::Prepare();
}


/**** M O U S E   M E T H O D S ****/


/******************************************************************************
 DispatchClick {OVERRIDE}

		Mouse click occurred inside a Window. Convert the mouse location
		in the mouse down event record from Global to Window coordinates.
		Then, let the superclass handle things.
 ******************************************************************************/

void	CWindow::DispatchClick(
	EventRecord		*macEvent)			/* Mouse down event record			*/
{
	Prepare();							/* Use Window's port and coords		*/
	GlobalToLocal(&macEvent->where);	/* Convert mouse location in event	*/
										/*   record to window coords		*/
	
	inherited::DispatchClick(macEvent);	/* Let superclass process the event	*/							
}
	

/******************************************************************************
 DispatchCursor {OVERRIDE}

		Find subview which wants to set the shape of the cursor. Point
		is passed in Global coordinates. Convert it to Window coordinates,
		then let superclass handle things.
 ******************************************************************************/

void	CWindow::DispatchCursor(
	Point		where,					/* Mouse location in global coords	*/
	RgnHandle	mouseRgn)				/* Region containing the mouse		*/
{
	Point		wTopLeft;
										
										/* Convert point to Window coords	*/
	wTopLeft = topLeft((**((WindowPeek)macPort)->contRgn).rgnBBox);
	where.h -= wTopLeft.h;
	where.v -= wTopLeft.v;
										/* Let superclass take over			*/
	inherited::DispatchCursor(where, mouseRgn);			
}

	
/******************************************************************************
 FrameToGlobalR {OVERRIDE}

		Convert a rectangle from Frame to Global coordinates
 ******************************************************************************/

void	CWindow::FrameToGlobalR(
	LongRect *frameRect, Rect	*globalRect)
{
	Point		offset;
	
	offset = topLeft((**((WindowPeek)macPort)->contRgn).rgnBBox);
	
	LongToQDRect( frameRect, globalRect);
	OffsetRect( globalRect, offset.h, offset.v);
}
