/******************************************************************************
 CDialogText.c

		Subclass of CEditText that implements text items in dialogs.
		DialogText items have a border, pass the special keys tab, enter,
		return, and escape to their supervisors, and can be validated.
		
	SUPERCLASS = CEditText
	
	Copyright  1991 Symantec Corporation. All rights reserved.
	

 ******************************************************************************/

#include "CDialogText.h"
#include "CTextEditTask.h"
#include "CDialog.h"
#include "CPaneBorder.h"
#include "Constants.h"
#include "TBUtilities.h"
#include "CBartender.h"
#include "Commands.h"
#include "Global.h"
#include <Packages.h>

#define kBorderAmount	2	/* white space between border and text of edit field */

extern CBartender	*gBartender;

/******************************************************************************
 IDialogText
 
 	Initialize a DialogText object. It is initialized to not required,
 	no maximum length, and not stylable.  Use the SetConstraints and Specify
 	methods to modify these characteristics.
 	
******************************************************************************/

void CDialogText::IDialogText(CView *anEnclosure, CView *aSupervisor,
				short aWidth, short aHeight,
				short aHEncl, short aVEncl,
				SizingOption aHSizing, SizingOption aVSizing,
				short aLineWidth)
{	
		
	CEditText::IEditText( anEnclosure, aSupervisor,
		aWidth, aHeight, aHEncl, aVEncl,
		aHSizing, aVSizing, aLineWidth);
		
	isRequired = FALSE;
	maxValidLength = MAXLONG;
	stylable = FALSE;
	validateOnResign = TRUE;

	IDialogTextX();

}	/* CDialogText::IDialogText */

/******************************************************************************
 IViewTemp {OVERRIDE}
 
 	Initialize a DialogText object from a resource.
******************************************************************************/

void CDialogText::IViewTemp(CView *anEnclosure, CBureaucrat *aSupervisor,
							Ptr viewData)
{
	tDialogTextTempP	template = (tDialogTextTempP) viewData;
	
	inherited::IViewTemp( anEnclosure, aSupervisor, viewData);
	
	isRequired = template->fRequired != 0;
	maxValidLength = template->maxLength;
	validateOnResign = template->fValidateOnResign != 0;
	
	IDialogTextX();

}	/* CDialogText::IViewTemp */

/******************************************************************************
 IDialogTextX
 
 	Do additional initialization for DialogText objects.
******************************************************************************/

void CDialogText::IDialogTextX( void)
{
	MakeBorder();
	SetWholeLines( FALSE);

}	/* CDialogText::IDialogTextX */

/******************************************************************************
 MakeBorder
 
 	Make a PaneBorder object for the DialogText. Override to change
 	the border appearance.
******************************************************************************/

void CDialogText::MakeBorder( void)
{
	Rect	margin;
	CPaneBorder *border;
	
	border = new( CPaneBorder);
	border->IPaneBorder( kBorderFrame);
	SetRect( &margin, -kBorderAmount, -kBorderAmount, kBorderAmount, kBorderAmount);
	border->SetMargin( &margin);
	SetBorder( border);

}	/* CDialogText::MakeBorder */

/******************************************************************************
 SetConstraints
 
 	Set the validation constraints for a DialogText object. IsRequired
 	determines if the text must be non-empty to be valid. maxValidLength
 	specifies the maximum number of characters the text may hold.
******************************************************************************/

void CDialogText::SetConstraints( Boolean fRequired, long aMaxChars)
{
	isRequired = fRequired;
	maxValidLength = aMaxChars;

}	/* CDialogText::SetConstraints */

/******************************************************************************
 DoKeyDown {OVERRIDE}
 
 	Handle keypresses in a DialogText object. Keys that have a special meaning
 	in a dialog (tab, return, enter, and Escape) are passed to itsSupervisor,
 	all other keys are passed through to the superclass.
******************************************************************************/

void CDialogText::DoKeyDown( char theChar, Byte keyCode, EventRecord *macEvent)
{
	Boolean pass = TRUE;
	short	ID;
	
	switch (theChar)
	{
		case '\t':
		case '\r':
		case kEnterKey:
			pass = FALSE;
			break;
			
		case kEscapeOrClear:
			if (keyCode == KeyEscape) pass = FALSE;
			break;							
	}
	if (pass)
	{
		inherited::DoKeyDown( theChar, keyCode, macEvent);
					
		if (itsTypingTask && itsTypingTask->CanStillType())
		{
			ID = this->ID;
			BroadcastChange( dialogTextChanged, &ID);
		}
	}
	else
		itsSupervisor->DoKeyDown( theChar, keyCode, macEvent);	

}	/* CDialogText::DoKeyDown */

/******************************************************************************
 GetTextString
 
 	Return the current contents as a Pascal string. If the text is longer
 	than 255 chars, then only the first 255 are returned.
******************************************************************************/

void CDialogText::GetTextString( StringPtr aString)
{
	short length = Min( (**macTE).teLength, 255);
	
	BlockMove( *(**macTE).hText, &aString[1], length);
	aString[0] = length;

}	/* CDialogText::GetTextString */

/******************************************************************************
 Validate
 
 	Validates that the currently specified constraints have been met.
 	This are controlled by the isRequired and maxValidLength instance
 	variables. Returns TRUE if the text is valid. Override to add
 	additional validation.
 	
******************************************************************************/

Boolean CDialogText::Validate( void)
{
	long textLength = GetLength();
	Boolean valid = TRUE;
	Str63	str;
	
	if (isRequired && (textLength == 0))
	{
		valid = FALSE;
		ReportInvalidText( validateFieldIsRequired);
	}
	else if (textLength > maxValidLength)
	{
		valid = FALSE;
		NumToString( maxValidLength, str);
		ParamText( NULL, str, NULL, NULL);
		ReportInvalidText( validateFieldTooLong);
	}
	return valid;

}	/* CDialogText::Validate */

/******************************************************************************
 ReportInvalidText
 
 	Handle reporting invalid text to the user. 
******************************************************************************/

void CDialogText::ReportInvalidText( short strIndex)
{
	Str255	message;
	
	SelectAll( TRUE);
	
	GetIndString( message, STRdlgValidation, strIndex);
	ParamText( message, NULL, NULL, NULL);
	PositionDialog('ALRT', ALRTvalidation);
	InitCursor();
	Alert( ALRTvalidation, NULL);
		

}	/* CDialogText::ReportInvalidText */

/******************************************************************************
 BecomeGopher
 
 	If the text is in a modal window, then enable and disable
 	the edit menu appropriately.
******************************************************************************/

Boolean CDialogText::BecomeGopher( Boolean fBecoming)
{	
	CWindow *window;
	Boolean windowVisible;
		
	window = GetWindow();
	windowVisible = window->IsVisible();
	
		/*
		 * if we are resigning the gopher and validation is requested
		 * then do the validation before calling inherited method
		 * note that we don't validate if window is not visible
		 */
		
	if (!fBecoming  && windowVisible && validateOnResign && IsActive() && !Validate())
		return FALSE;
		
	if (!inherited::BecomeGopher(fBecoming))
		return FALSE;
		
	if (windowVisible && window->IsModal())
	{
		if (fBecoming)
			gBartender->EnableMenu( MENUedit);
		else
			gBartender->DisableMenu( MENUedit);
	}
	
	return TRUE;

}	/* CDialogText::BecomeGopher */