/******************************************************************************
 CSelector.c

							The Selector Class
		
	Abstract class for a group of items from which a single selection can
	be made, usually with the mouse.
	
	SUPERCLASS = CPanorama
	
	Copyright  1989 Symantec Corporation. All rights reserved.

 ******************************************************************************/

#include "Global.h"
#include "CSelector.h"

/*** Global Variables ***/

extern short		gClicks;			/* Click counter, = 1 single click	*/
										/*                = 2 double click	*/
										/*    etc.							*/
	

/******************************************************************************
 ISelector

		Initialize a Selector object
 ******************************************************************************/

void	CSelector::ISelector(
	CView			*anEnclosure,
	CBureaucrat		*aSupervisor,
	short			aWidth,
	short			aHeight,
	short			aHEncl,
	short			aVEncl,
	SizingOption	aHSizing,
	SizingOption	aVSizing,
	short			aNumItems,
	short			aSelection,
	short			aCommandBase)
{
	CPanorama::IPanorama(anEnclosure, aSupervisor,
					aWidth, aHeight, aHEncl, aVEncl, aHSizing, aVSizing);
	
	wantsClicks = TRUE;
	numItems = aNumItems;
	selection = aSelection;
	commandBase = aCommandBase;
}
	

/******************************************************************************
 DoClick {OVERRIDE}

		User clicked inside the selector. Turn of hiliting of the
		newly selected item and unhilite the former selection. If
		click is in the current selection, check for a double click.
		Note that triple (and greater) clicks are ignored.
 ******************************************************************************/

void	CSelector::DoClick(
	Point		hitPt,
	short		modifierKeys,
	long		when)
{
	short			itemHit;			/* Index no. of clicked item		*/
	
	itemHit = FindItem( hitPt);			/* Determine item clicked on		*/
	
	if (itemHit != selection) {			/* If this is a new selection ...	*/
		ChangeSelection(itemHit);		/*   Change the current selection	*/
										/*   Send command to supervisor		*/
		itsSupervisor->DoCommand(-(((long)commandBase << 16) + itemHit));
		
	} else if (gClicks == 2) {			/* Item clicked is already the		*/
		DoDoubleClick();				/*   current selection. See if it's	*/
										/*   a double click.				*/
	}
}
	

/******************************************************************************
 HitSamePart {OVERRIDE}

		Check whether two points hit the same item in a Selector.
		Points are in Window coordinates.
 ******************************************************************************/

Boolean		CSelector::HitSamePart(
	Point			pointA,
	Point			pointB)
{
	return( FindItem( pointA) == FindItem( pointB) );
}
	

/******************************************************************************
 ChangeSelection

		Select a new item by unhiliting the former selection and
		hiliting the new item
 ******************************************************************************/

void	CSelector::ChangeSelection(
	short	aSelection)
{
	if (aSelection != selection) {
		if (visible) {
			HiliteItem(selection, hiliteOFF);
			HiliteItem(aSelection, hiliteON);
		}
		selection = aSelection;
	}
}
	

/******************************************************************************
 GetSelection

		Return the number of the currently selected item
 ******************************************************************************/

short	CSelector::GetSelection()
{
	return(selection);					/* Return instance variable			*/
}
	

/******************************************************************************
 SetCommandBase

		Specify the value of the command base. When an item is chosen
		the command number has the command base in the Hi word and the
		item number in the Lo word.
 ******************************************************************************/

void	CSelector::SetCommandBase(
	short		aCommandBase)
{
	commandBase = aCommandBase;
}
	

/******************************************************************************
 GetCommandBase

		Return the value of the command base. When an item is chosen
		the command number has the command base in the Hi word and the
		item number in the Lo word.
 ******************************************************************************/

short	CSelector::GetCommandBase()
{
	return(commandBase);				/* Return instance variable			*/
}
	

/******************************************************************************
 HiliteItem

		Hilite an item in the specified way. Subclasses must override
		this method.
 ******************************************************************************/

void	CSelector::HiliteItem(
	short		theItem,				/* Item number to hilite			*/
	HiliteState	state)					/* State of hiliting desired		*/
{
}										/* Null Method						*/
	

/******************************************************************************
 FindItem

		Determine which item, if any, is selected by a click at the
		specified hit point. Subclasses must override this method.
 ******************************************************************************/

short	CSelector::FindItem(
	Point		hitPt)					/* Click location in pane coords	*/
{
	return(NOTHING);
}
	

/******************************************************************************
 DoDoubleClick

		Respond to a double-click on an item in a Selector. The first
		click will have set the selection, so the item double-clicked
		must be the current selection. Subclasses which support double
		clicks must override this method.
 ******************************************************************************/

void	CSelector::DoDoubleClick()
{
}										/* Null Method						*/